@@ -2,49 +2,67 @@
 
   class.c -
 
-  $Author: shyouhei $
-  $Date: 2009-01-15 18:58:45 -0700 (Thu, 15 Jan 2009) $
+  $Author: akr $
   created at: Tue Aug 10 15:05:44 JST 1993
 
-  Copyright (C) 1993-2003 Yukihiro Matsumoto
+  Copyright (C) 1993-2007 Yukihiro Matsumoto
 
 **********************************************************************/
 
-#include "ruby.h"
-#include "rubysig.h"
-#include "node.h"
-#include "st.h"
+#include "ruby/ruby.h"
+#include "ruby/signal.h"
+#include "ruby/node.h"
+#include "ruby/st.h"
 #include <ctype.h>
 
 extern st_table *rb_class_tbl;
 
-VALUE
-rb_class_boot(super)
-    VALUE super;
+static VALUE
+class_alloc(VALUE flags, VALUE klass)
 {
-    NEWOBJ(klass, struct RClass);
-    OBJSETUP(klass, rb_cClass, T_CLASS);
+    rb_classext_t *ext = ALLOC(rb_classext_t);
+    NEWOBJ(obj, struct RClass);
+    OBJSETUP(obj, klass, flags);
+    obj->ptr = ext;
+    RCLASS_IV_TBL(obj) = 0;
+    RCLASS_M_TBL(obj) = 0;
+    RCLASS_SUPER(obj) = 0;
+    RCLASS_IV_INDEX_TBL(obj) = 0;
+    return (VALUE)obj;
+}
 
-    klass->super = super;
-    klass->iv_tbl = 0;
-    klass->m_tbl = 0;		/* safe GC */
-    klass->m_tbl = st_init_numtable();
+VALUE
+rb_class_boot(VALUE super)
+{
+    VALUE klass = class_alloc(T_CLASS, rb_cClass);
+
+    RCLASS_SUPER(klass) = super;
+    RCLASS_M_TBL(klass) = st_init_numtable();
 
     OBJ_INFECT(klass, super);
     return (VALUE)klass;
 }
 
+void
+rb_check_inheritable(VALUE super)
+{
+    if (TYPE(super) != T_CLASS) {
+	rb_raise(rb_eTypeError, "superclass must be a Class (%s given)",
+		 rb_obj_classname(super));
+    }
+    if (RBASIC(super)->flags & FL_SINGLETON) {
+	rb_raise(rb_eTypeError, "can't make subclass of singleton class");
+    }
+}
+
 VALUE
-rb_class_new(super)
-    VALUE super;
+rb_class_new(VALUE super)
 {
     Check_Type(super, T_CLASS);
+    rb_check_inheritable(super);
     if (super == rb_cClass) {
 	rb_raise(rb_eTypeError, "can't make subclass of Class");
     }
-    if (FL_TEST(super, FL_SINGLETON)) {
-	rb_raise(rb_eTypeError, "can't make subclass of virtual class");
-    }
     return rb_class_boot(super);
 }
 
@@ -54,50 +72,47 @@ struct clone_method_data {
 };
 
 static int
-clone_method(mid, body, data)
-    ID mid;
-    NODE *body;
-    struct clone_method_data *data;
+clone_method(ID mid, NODE *body, struct clone_method_data *data)
 {
-    NODE *fbody = body->nd_body;
-
-    if (fbody && nd_type(fbody) == NODE_SCOPE) {
-	NODE *cref = (NODE*)fbody->nd_rval;
-
-	if (cref) cref = cref->nd_next;
-	fbody = rb_copy_node_scope(fbody, NEW_CREF(data->klass, cref));
+    if (body == 0) {
+	st_insert(data->tbl, mid, 0);
+    }
+    else {
+	st_insert(data->tbl, mid,
+		  (st_data_t)
+		  NEW_FBODY(
+		      NEW_METHOD(body->nd_body->nd_body,
+				 data->klass, /* TODO */
+				 body->nd_body->nd_noex),
+		      0));
     }
-    st_insert(data->tbl, mid, (st_data_t)NEW_METHOD(fbody, body->nd_noex));
     return ST_CONTINUE;
 }
 
 /* :nodoc: */
 VALUE
-rb_mod_init_copy(clone, orig)
-    VALUE clone, orig;
+rb_mod_init_copy(VALUE clone, VALUE orig)
 {
     rb_obj_init_copy(clone, orig);
     if (!FL_TEST(CLASS_OF(clone), FL_SINGLETON)) {
-	RBASIC(clone)->klass = RBASIC(orig)->klass;
-	RBASIC(clone)->klass = rb_singleton_class_clone(clone);
+	RBASIC(clone)->klass = rb_singleton_class_clone(orig);
     }
-    RCLASS(clone)->super = RCLASS(orig)->super;
-    if (RCLASS(orig)->iv_tbl) {
+    RCLASS_SUPER(clone) = RCLASS_SUPER(orig);
+    if (RCLASS_IV_TBL(orig)) {
 	ID id;
 
-	RCLASS(clone)->iv_tbl = st_copy(RCLASS(orig)->iv_tbl);
+	RCLASS_IV_TBL(clone) = st_copy(RCLASS_IV_TBL(orig));
 	id = rb_intern("__classpath__");
-	st_delete(RCLASS(clone)->iv_tbl, (st_data_t*)&id, 0);
+	st_delete(RCLASS_IV_TBL(clone), (st_data_t*)&id, 0);
 	id = rb_intern("__classid__");
-	st_delete(RCLASS(clone)->iv_tbl, (st_data_t*)&id, 0);
+	st_delete(RCLASS_IV_TBL(clone), (st_data_t*)&id, 0);
     }
-    if (RCLASS(orig)->m_tbl) {
+    if (RCLASS_M_TBL(orig)) {
 	struct clone_method_data data;
-
-	data.tbl = RCLASS(clone)->m_tbl = st_init_numtable();
-	data.klass = (VALUE)clone;
-
-	st_foreach(RCLASS(orig)->m_tbl, clone_method, (st_data_t)&data);
+	data.tbl = RCLASS_M_TBL(clone) = st_init_numtable();
+	data.klass = clone;
+	st_foreach(RCLASS_M_TBL(orig), clone_method,
+	  (st_data_t)&data);
     }
 
     return clone;
@@ -105,10 +120,9 @@ rb_mod_init_copy(clone, orig)
 
 /* :nodoc: */
 VALUE
-rb_class_init_copy(clone, orig)
-    VALUE clone, orig;
+rb_class_init_copy(VALUE clone, VALUE orig)
 {
-    if (RCLASS(clone)->super != 0) {
+    if (RCLASS_SUPER(clone) != 0) {
 	rb_raise(rb_eTypeError, "already initialized class");
     }
     if (FL_TEST(orig, FL_SINGLETON)) {
@@ -118,17 +132,16 @@ rb_class_init_copy(clone, orig)
 }
 
 VALUE
-rb_singleton_class_clone(obj)
-    VALUE obj;
+rb_singleton_class_clone(VALUE obj)
 {
     VALUE klass = RBASIC(obj)->klass;
 
     if (!FL_TEST(klass, FL_SINGLETON))
 	return klass;
     else {
+	struct clone_method_data data;
 	/* copy singleton(unnamed) class */
-	NEWOBJ(clone, struct RClass);
-	OBJSETUP(clone, 0, RBASIC(klass)->flags);
+        VALUE clone = class_alloc(RBASIC(klass)->flags, 0);
 
 	if (BUILTIN_TYPE(obj) == T_CLASS) {
 	    RBASIC(clone)->klass = (VALUE)clone;
@@ -137,28 +150,15 @@ rb_singleton_class_clone(obj)
 	    RBASIC(clone)->klass = rb_singleton_class_clone(klass);
 	}
 
-	clone->super = RCLASS(klass)->super;
-	clone->iv_tbl = 0;
-	clone->m_tbl = 0;
-	if (RCLASS(klass)->iv_tbl) {
-	    clone->iv_tbl = st_copy(RCLASS(klass)->iv_tbl);
-	}
-	{
-	    struct clone_method_data data;
-
-	    data.tbl = clone->m_tbl = st_init_numtable();
-	    switch (TYPE(obj)) {
-	      case T_CLASS:
-	      case T_MODULE:
-		data.klass = obj;
-		break;
-	      default:
-		data.klass = 0;
-		break;
-	    }
-
-	    st_foreach(RCLASS(klass)->m_tbl, clone_method, (st_data_t)&data);
+	RCLASS_SUPER(clone) = RCLASS_SUPER(klass);
+	if (RCLASS_IV_TBL(klass)) {
+	    RCLASS_IV_TBL(clone) = st_copy(RCLASS_IV_TBL(klass));
 	}
+	RCLASS_M_TBL(clone) = st_init_numtable();
+	data.tbl = RCLASS_M_TBL(clone);
+	data.klass = (VALUE)clone;
+	st_foreach(RCLASS_M_TBL(klass), clone_method,
+	  (st_data_t)&data);
 	rb_singleton_class_attached(RBASIC(clone)->klass, (VALUE)clone);
 	FL_SET(clone, FL_SINGLETON);
 	return (VALUE)clone;
@@ -166,45 +166,41 @@ rb_singleton_class_clone(obj)
 }
 
 void
-rb_singleton_class_attached(klass, obj)
-    VALUE klass, obj;
+rb_singleton_class_attached(VALUE klass, VALUE obj)
 {
     if (FL_TEST(klass, FL_SINGLETON)) {
-	if (!RCLASS(klass)->iv_tbl) {
-	    RCLASS(klass)->iv_tbl = st_init_numtable();
+	if (!RCLASS_IV_TBL(klass)) {
+	    RCLASS_IV_TBL(klass) = st_init_numtable();
 	}
-	st_insert(RCLASS(klass)->iv_tbl, rb_intern("__attached__"), obj);
+	st_insert(RCLASS_IV_TBL(klass), rb_intern("__attached__"), obj);
     }
 }
 
 VALUE
-rb_make_metaclass(obj, super)
-    VALUE obj, super;
+rb_make_metaclass(VALUE obj, VALUE super)
 {
-    VALUE klass = rb_class_boot(super);
-    FL_SET(klass, FL_SINGLETON);
-    RBASIC(obj)->klass = klass;
-    rb_singleton_class_attached(klass, obj);
     if (BUILTIN_TYPE(obj) == T_CLASS && FL_TEST(obj, FL_SINGLETON)) {
-	RBASIC(klass)->klass = klass;
-	RCLASS(klass)->super = RBASIC(rb_class_real(RCLASS(obj)->super))->klass;
+	return RBASIC(obj)->klass = rb_cClass;
     }
     else {
-	VALUE metasuper = RBASIC(rb_class_real(super))->klass;
+	VALUE metasuper;
+	VALUE klass = rb_class_boot(super);
 
+	FL_SET(klass, FL_SINGLETON);
+	RBASIC(obj)->klass = klass;
+	rb_singleton_class_attached(klass, obj);
+
+	metasuper = RBASIC(rb_class_real(super))->klass;
 	/* metaclass of a superclass may be NULL at boot time */
 	if (metasuper) {
 	    RBASIC(klass)->klass = metasuper;
 	}
+	return klass;
     }
-
-    return klass;
 }
 
 VALUE
-rb_define_class_id(id, super)
-    ID id;
-    VALUE super;
+rb_define_class_id(ID id, VALUE super)
 {
     VALUE klass;
 
@@ -215,31 +211,15 @@ rb_define_class_id(id, super)
     return klass;
 }
 
-void
-rb_check_inheritable(super)
-    VALUE super;
-{
-    if (TYPE(super) != T_CLASS) {
-	rb_raise(rb_eTypeError, "superclass must be a Class (%s given)",
-		 rb_obj_classname(super));
-    }
-    if (RBASIC(super)->flags & FL_SINGLETON) {
-	rb_raise(rb_eTypeError, "can't make subclass of virtual class");
-    }
-}
-
 VALUE
-rb_class_inherited(super, klass)
-    VALUE super, klass;
+rb_class_inherited(VALUE super, VALUE klass)
 {
     if (!super) super = rb_cObject;
     return rb_funcall(super, rb_intern("inherited"), 1, klass);
 }
 
 VALUE
-rb_define_class(name, super)
-    const char *name;
-    VALUE super;
+rb_define_class(const char *name, VALUE super)
 {
     VALUE klass;
     ID id;
@@ -250,7 +230,7 @@ rb_define_class(name, super)
 	if (TYPE(klass) != T_CLASS) {
 	    rb_raise(rb_eTypeError, "%s is not a class", name);
 	}
-	if (rb_class_real(RCLASS(klass)->super) != super) {
+	if (rb_class_real(RCLASS_SUPER(klass)) != super) {
 	    rb_name_error(id, "%s is already defined", name);
 	}
 	return klass;
@@ -268,10 +248,7 @@ rb_define_class(name, super)
 }
 
 VALUE
-rb_define_class_under(outer, name, super)
-    VALUE outer;
-    const char *name;
-    VALUE super;
+rb_define_class_under(VALUE outer, const char *name, VALUE super)
 {
     VALUE klass;
     ID id;
@@ -282,7 +259,7 @@ rb_define_class_under(outer, name, super)
 	if (TYPE(klass) != T_CLASS) {
 	    rb_raise(rb_eTypeError, "%s is not a class", name);
 	}
-	if (rb_class_real(RCLASS(klass)->super) != super) {
+	if (rb_class_real(RCLASS_SUPER(klass)) != super) {
 	    rb_name_error(id, "%s is already defined", name);
 	}
 	return klass;
@@ -300,22 +277,17 @@ rb_define_class_under(outer, name, super)
 }
 
 VALUE
-rb_module_new()
+rb_module_new(void)
 {
-    NEWOBJ(mdl, struct RClass);
-    OBJSETUP(mdl, rb_cModule, T_MODULE);
+    VALUE mdl = class_alloc(T_MODULE, rb_cModule);
 
-    mdl->super = 0;
-    mdl->iv_tbl = 0;
-    mdl->m_tbl = 0;
-    mdl->m_tbl = st_init_numtable();
+    RCLASS_M_TBL(mdl) = st_init_numtable();
 
     return (VALUE)mdl;
 }
 
 VALUE
-rb_define_module_id(id)
-    ID id;
+rb_define_module_id(ID id)
 {
     VALUE mdl;
 
@@ -326,8 +298,7 @@ rb_define_module_id(id)
 }
 
 VALUE
-rb_define_module(name)
-    const char *name;
+rb_define_module(const char *name)
 {
     VALUE module;
     ID id;
@@ -347,9 +318,7 @@ rb_define_module(name)
 }
 
 VALUE
-rb_define_module_under(outer, name)
-    VALUE outer;
-    const char *name;
+rb_define_module_under(VALUE outer, const char *name)
 {
     VALUE module;
     ID id;
@@ -370,21 +339,19 @@ rb_define_module_under(outer, name)
 }
 
 static VALUE
-include_class_new(module, super)
-    VALUE module, super;
+include_class_new(VALUE module, VALUE super)
 {
-    NEWOBJ(klass, struct RClass);
-    OBJSETUP(klass, rb_cClass, T_ICLASS);
+    VALUE klass = class_alloc(T_ICLASS, rb_cClass);
 
     if (BUILTIN_TYPE(module) == T_ICLASS) {
 	module = RBASIC(module)->klass;
     }
-    if (!RCLASS(module)->iv_tbl) {
-	RCLASS(module)->iv_tbl = st_init_numtable();
+    if (!RCLASS_IV_TBL(module)) {
+	RCLASS_IV_TBL(module) = st_init_numtable();
     }
-    klass->iv_tbl = RCLASS(module)->iv_tbl;
-    klass->m_tbl = RCLASS(module)->m_tbl;
-    klass->super = super;
+    RCLASS_IV_TBL(klass) = RCLASS_IV_TBL(module);
+    RCLASS_M_TBL(klass) = RCLASS_M_TBL(module);
+    RCLASS_SUPER(klass) = super;
     if (TYPE(module) == T_ICLASS) {
 	RBASIC(klass)->klass = RBASIC(module)->klass;
     }
@@ -398,8 +365,7 @@ include_class_new(module, super)
 }
 
 void
-rb_include_module(klass, module)
-    VALUE klass, module;
+rb_include_module(VALUE klass, VALUE module)
 {
     VALUE p, c;
     int changed = 0;
@@ -416,30 +382,30 @@ rb_include_module(klass, module)
     OBJ_INFECT(klass, module);
     c = klass;
     while (module) {
-	int superclass_seen = Qfalse;
+       int superclass_seen = Qfalse;
 
-	if (RCLASS(klass)->m_tbl == RCLASS(module)->m_tbl)
+	if (RCLASS_M_TBL(klass) == RCLASS_M_TBL(module))
 	    rb_raise(rb_eArgError, "cyclic include detected");
-	/* ignore if the module included already in superclasses */
-	for (p = RCLASS(klass)->super; p; p = RCLASS(p)->super) {
-	    switch (BUILTIN_TYPE(p)) {
-	      case T_ICLASS:
-		if (RCLASS(p)->m_tbl == RCLASS(module)->m_tbl) {
-		    if (!superclass_seen) {
-			c = p;	/* move insertion point */
-		    }
-		    goto skip;
-		}
-		break;
-	      case T_CLASS:
-		superclass_seen = Qtrue;
-		break;
-	    }
-	}
-	c = RCLASS(c)->super = include_class_new(module, RCLASS(c)->super);
+       /* ignore if the module included already in superclasses */
+       for (p = RCLASS_SUPER(klass); p; p = RCLASS_SUPER(p)) {
+           switch (BUILTIN_TYPE(p)) {
+             case T_ICLASS:
+               if (RCLASS_M_TBL(p) == RCLASS_M_TBL(module)) {
+                   if (!superclass_seen) {
+                       c = p;  /* move insertion point */
+                   }
+                   goto skip;
+               }
+               break;
+             case T_CLASS:
+               superclass_seen = Qtrue;
+               break;
+           }
+       }
+       c = RCLASS_SUPER(c) = include_class_new(module, RCLASS_SUPER(c));
 	changed = 1;
       skip:
-	module = RCLASS(module)->super;
+	module = RCLASS_SUPER(module);
     }
     if (changed) rb_clear_cache();
 }
@@ -462,13 +428,12 @@ rb_include_module(klass, module)
  */
 
 VALUE
-rb_mod_included_modules(mod)
-    VALUE mod;
+rb_mod_included_modules(VALUE mod)
 {
     VALUE ary = rb_ary_new();
     VALUE p;
 
-    for (p = RCLASS(mod)->super; p; p = RCLASS(p)->super) {
+    for (p = RCLASS_SUPER(mod); p; p = RCLASS_SUPER(p)) {
 	if (BUILTIN_TYPE(p) == T_ICLASS) {
 	    rb_ary_push(ary, RBASIC(p)->klass);
 	}
@@ -496,14 +461,12 @@ rb_mod_included_modules(mod)
  */
 
 VALUE
-rb_mod_include_p(mod, mod2)
-    VALUE mod;
-    VALUE mod2;
+rb_mod_include_p(VALUE mod, VALUE mod2)
 {
     VALUE p;
 
     Check_Type(mod2, T_MODULE);
-    for (p = RCLASS(mod)->super; p; p = RCLASS(p)->super) {
+    for (p = RCLASS_SUPER(mod); p; p = RCLASS_SUPER(p)) {
 	if (BUILTIN_TYPE(p) == T_ICLASS) {
 	    if (RBASIC(p)->klass == mod2) return Qtrue;
 	}
@@ -528,12 +491,11 @@ rb_mod_include_p(mod, mod2)
  */
 
 VALUE
-rb_mod_ancestors(mod)
-    VALUE mod;
+rb_mod_ancestors(VALUE mod)
 {
     VALUE p, ary = rb_ary_new();
 
-    for (p = mod; p; p = RCLASS(p)->super) {
+    for (p = mod; p; p = RCLASS_SUPER(p)) {
 	if (FL_TEST(p, FL_SINGLETON))
 	    continue;
 	if (BUILTIN_TYPE(p) == T_ICLASS) {
@@ -550,13 +512,10 @@ rb_mod_ancestors(mod)
 #define VISI_CHECK(x,f) (VISI(x) == (f))
 
 static int
-ins_methods_push(name, type, ary, visi)
-    ID name;
-    long type;
-    VALUE ary;
-    long visi;
+ins_methods_push(ID name, long type, VALUE ary, long visi)
 {
     if (type == -1) return ST_CONTINUE;
+
     switch (visi) {
       case NOEX_PRIVATE:
       case NOEX_PROTECTED:
@@ -568,70 +527,58 @@ ins_methods_push(name, type, ary, visi)
 	break;
     }
     if (visi) {
-	rb_ary_push(ary, rb_str_new2(rb_id2name(name)));
+	rb_ary_push(ary, ID2SYM(name));
     }
     return ST_CONTINUE;
 }
 
 static int
-ins_methods_i(name, type, ary)
-    ID name;
-    long type;
-    VALUE ary;
+ins_methods_i(ID name, long type, VALUE ary)
 {
     return ins_methods_push(name, type, ary, -1); /* everything but private */
 }
 
 static int
-ins_methods_prot_i(name, type, ary)
-    ID name;
-    long type;
-    VALUE ary;
+ins_methods_prot_i(ID name, long type, VALUE ary)
 {
     return ins_methods_push(name, type, ary, NOEX_PROTECTED);
 }
 
 static int
-ins_methods_priv_i(name, type, ary)
-    ID name;
-    long type;
-    VALUE ary;
+ins_methods_priv_i(ID name, long type, VALUE ary)
 {
     return ins_methods_push(name, type, ary, NOEX_PRIVATE);
 }
 
 static int
-ins_methods_pub_i(name, type, ary)
-    ID name;
-    long type;
-    VALUE ary;
+ins_methods_pub_i(ID name, long type, VALUE ary)
 {
     return ins_methods_push(name, type, ary, NOEX_PUBLIC);
 }
 
 static int
-method_entry(key, body, list)
-    ID key;
-    NODE *body;
-    st_table *list;
+method_entry(ID key, NODE *body, st_table *list)
 {
     long type;
 
-    if (key == ID_ALLOCATOR) return ST_CONTINUE;
+    if (key == ID_ALLOCATOR) {
+	return ST_CONTINUE;
+    }
+    
     if (!st_lookup(list, key, 0)) {
-	if (!body->nd_body) type = -1; /* none */
-	else type = VISI(body->nd_noex);
+	if (body ==0 || !body->nd_body->nd_body) {
+	    type = -1; /* none */
+	}
+	else {
+	    type = VISI(body->nd_body->nd_noex);
+	}
 	st_add_direct(list, key, type);
     }
     return ST_CONTINUE;
 }
 
 static VALUE
-class_instance_method_list(argc, argv, mod, func)
-    int argc;
-    VALUE *argv;
-    VALUE mod;
-    int (*func) _((ID, long, VALUE));
+class_instance_method_list(int argc, VALUE *argv, VALUE mod, int (*func) (ID, long, VALUE))
 {
     VALUE ary;
     int recur;
@@ -647,8 +594,8 @@ class_instance_method_list(argc, argv, mod, func)
     }
 
     list = st_init_numtable();
-    for (; mod; mod = RCLASS(mod)->super) {
-	st_foreach(RCLASS(mod)->m_tbl, method_entry, (st_data_t)list);
+    for (; mod; mod = RCLASS_SUPER(mod)) {
+	st_foreach(RCLASS_M_TBL(mod), method_entry, (st_data_t)list);
 	if (BUILTIN_TYPE(mod) == T_ICLASS) continue;
 	if (FL_TEST(mod, FL_SINGLETON)) continue;
 	if (!recur) break;
@@ -688,10 +635,7 @@ class_instance_method_list(argc, argv, mod, func)
  */
 
 VALUE
-rb_class_instance_methods(argc, argv, mod)
-    int argc;
-    VALUE *argv;
-    VALUE mod;
+rb_class_instance_methods(int argc, VALUE *argv, VALUE mod)
 {
     return class_instance_method_list(argc, argv, mod, ins_methods_i);
 }
@@ -706,10 +650,7 @@ rb_class_instance_methods(argc, argv, mod)
  */
 
 VALUE
-rb_class_protected_instance_methods(argc, argv, mod)
-    int argc;
-    VALUE *argv;
-    VALUE mod;
+rb_class_protected_instance_methods(int argc, VALUE *argv, VALUE mod)
 {
     return class_instance_method_list(argc, argv, mod, ins_methods_prot_i);
 }
@@ -732,10 +673,7 @@ rb_class_protected_instance_methods(argc, argv, mod)
  */
 
 VALUE
-rb_class_private_instance_methods(argc, argv, mod)
-    int argc;
-    VALUE *argv;
-    VALUE mod;
+rb_class_private_instance_methods(int argc, VALUE *argv, VALUE mod)
 {
     return class_instance_method_list(argc, argv, mod, ins_methods_priv_i);
 }
@@ -750,10 +688,7 @@ rb_class_private_instance_methods(argc, argv, mod)
  */
 
 VALUE
-rb_class_public_instance_methods(argc, argv, mod)
-    int argc;
-    VALUE *argv;
-    VALUE mod;
+rb_class_public_instance_methods(int argc, VALUE *argv, VALUE mod)
 {
     return class_instance_method_list(argc, argv, mod, ins_methods_pub_i);
 }
@@ -791,10 +726,7 @@ rb_class_public_instance_methods(argc, argv, mod)
  */
 
 VALUE
-rb_obj_singleton_methods(argc, argv, obj)
-    int argc;
-    VALUE *argv;
-    VALUE obj;
+rb_obj_singleton_methods(int argc, VALUE *argv, VALUE obj)
 {
     VALUE recur, ary, klass;
     st_table *list;
@@ -806,13 +738,13 @@ rb_obj_singleton_methods(argc, argv, obj)
     klass = CLASS_OF(obj);
     list = st_init_numtable();
     if (klass && FL_TEST(klass, FL_SINGLETON)) {
-	st_foreach(RCLASS(klass)->m_tbl, method_entry, (st_data_t)list);
-	klass = RCLASS(klass)->super;
+	st_foreach(RCLASS_M_TBL(klass), method_entry, (st_data_t)list);
+	klass = RCLASS_SUPER(klass);
     }
     if (RTEST(recur)) {
 	while (klass && (FL_TEST(klass, FL_SINGLETON) || TYPE(klass) == T_ICLASS)) {
-	    st_foreach(RCLASS(klass)->m_tbl, method_entry, (st_data_t)list);
-	    klass = RCLASS(klass)->super;
+	    st_foreach(RCLASS_M_TBL(klass), method_entry, (st_data_t)list);
+	    klass = RCLASS_SUPER(klass);
 	}
     }
     ary = rb_ary_new();
@@ -823,53 +755,31 @@ rb_obj_singleton_methods(argc, argv, obj)
 }
 
 void
-rb_define_method_id(klass, name, func, argc)
-    VALUE klass;
-    ID name;
-    VALUE (*func)();
-    int argc;
+rb_define_method_id(VALUE klass, ID name, VALUE (*func)(ANYARGS), int argc)
 {
     rb_add_method(klass, name, NEW_CFUNC(func,argc), NOEX_PUBLIC);
 }
 
 void
-rb_define_method(klass, name, func, argc)
-    VALUE klass;
-    const char *name;
-    VALUE (*func)();
-    int argc;
+rb_define_method(VALUE klass, const char *name, VALUE (*func)(ANYARGS), int argc)
 {
-    ID id = rb_intern(name);
-    int ex = NOEX_PUBLIC;
-
-
-    rb_add_method(klass, id, NEW_CFUNC(func, argc), ex);
+    rb_add_method(klass, rb_intern(name), NEW_CFUNC(func, argc), NOEX_PUBLIC);
 }
 
 void
-rb_define_protected_method(klass, name, func, argc)
-    VALUE klass;
-    const char *name;
-    VALUE (*func)();
-    int argc;
+rb_define_protected_method(VALUE klass, const char *name, VALUE (*func)(ANYARGS), int argc)
 {
     rb_add_method(klass, rb_intern(name), NEW_CFUNC(func, argc), NOEX_PROTECTED);
 }
 
 void
-rb_define_private_method(klass, name, func, argc)
-    VALUE klass;
-    const char *name;
-    VALUE (*func)();
-    int argc;
+rb_define_private_method(VALUE klass, const char *name, VALUE (*func)(ANYARGS), int argc)
 {
     rb_add_method(klass, rb_intern(name), NEW_CFUNC(func, argc), NOEX_PRIVATE);
 }
 
 void
-rb_undef_method(klass, name)
-    VALUE klass;
-    const char *name;
+rb_undef_method(VALUE klass, const char *name)
 {
     rb_add_method(klass, rb_intern(name), 0, NOEX_UNDEF);
 }
@@ -881,8 +791,7 @@ rb_undef_method(klass, name)
 } while (0)
 
 VALUE
-rb_singleton_class(obj)
-    VALUE obj;
+rb_singleton_class(VALUE obj)
 {
     VALUE klass;
 
@@ -917,77 +826,47 @@ rb_singleton_class(obj)
 }
 
 void
-rb_define_singleton_method(obj, name, func, argc)
-    VALUE obj;
-    const char *name;
-    VALUE (*func)();
-    int argc;
+rb_define_singleton_method(VALUE obj, const char *name, VALUE (*func)(ANYARGS), int argc)
 {
     rb_define_method(rb_singleton_class(obj), name, func, argc);
 }
 
 void
-rb_define_module_function(module, name, func, argc)
-    VALUE module;
-    const char *name;
-    VALUE (*func)();
-    int argc;
+rb_define_module_function(VALUE module, const char *name, VALUE (*func)(ANYARGS), int argc)
 {
     rb_define_private_method(module, name, func, argc);
     rb_define_singleton_method(module, name, func, argc);
 }
 
 void
-rb_define_global_function(name, func, argc)
-    const char *name;
-    VALUE (*func)();
-    int argc;
+rb_define_global_function(const char *name, VALUE (*func)(ANYARGS), int argc)
 {
     rb_define_module_function(rb_mKernel, name, func, argc);
 }
 
 void
-rb_define_alias(klass, name1, name2)
-    VALUE klass;
-    const char *name1, *name2;
+rb_define_alias(VALUE klass, const char *name1, const char *name2)
 {
     rb_alias(klass, rb_intern(name1), rb_intern(name2));
 }
 
 void
-rb_define_attr(klass, name, read, write)
-    VALUE klass;
-    const char *name;
-    int read, write;
+rb_define_attr(VALUE klass, const char *name, int read, int write)
 {
     rb_attr(klass, rb_intern(name), read, write, Qfalse);
 }
 
-#ifdef HAVE_STDARG_PROTOTYPES
 #include <stdarg.h>
-#define va_init_list(a,b) va_start(a,b)
-#else
-#include <varargs.h>
-#define va_init_list(a,b) va_start(a)
-#endif
 
 int
-#ifdef HAVE_STDARG_PROTOTYPES
 rb_scan_args(int argc, const VALUE *argv, const char *fmt, ...)
-#else
-rb_scan_args(argc, argv, fmt, va_alist)
-    int argc;
-    const VALUE *argv;
-    const char *fmt;
-    va_dcl
-#endif
 {
     int n, i = 0;
     const char *p = fmt;
     VALUE *var;
     va_list vargs;
 
-    va_init_list(vargs, fmt);
+    va_start(vargs, fmt);
 
     if (*p == '*') goto rest_arg;
 
