@@ -2,8 +2,7 @@
 
   dbm.c -
 
-  $Author: shyouhei $
-  $Date: 2007-02-13 08:01:19 +0900 (Tue, 13 Feb 2007) $
+  $Author: akr $
   created at: Mon Jan 24 15:59:52 JST 1994
 
   Copyright (C) 1995-2001 Yukihiro Matsumoto
@@ -32,7 +31,7 @@ struct dbmdata {
 };
 
 static void
-closed_dbm()
+closed_dbm(void)
 {
     rb_raise(rb_eDBMError, "closed DBM file");
 }
@@ -49,8 +48,7 @@ closed_dbm()
 }
 
 static void
-free_dbm(dbmp)
-    struct dbmdata *dbmp;
+free_dbm(struct dbmdata *dbmp)
 {
     if (dbmp) {
 	if (dbmp->di_dbm) dbm_close(dbmp->di_dbm);
@@ -59,8 +57,7 @@ free_dbm(dbmp)
 }
 
 static VALUE
-fdbm_close(obj)
-    VALUE obj;
+fdbm_close(VALUE obj)
 {
     struct dbmdata *dbmp;
 
@@ -72,8 +69,7 @@ fdbm_close(obj)
 }
 
 static VALUE
-fdbm_closed(obj)
-    VALUE obj;
+fdbm_closed(VALUE obj)
 {
     struct dbmdata *dbmp;
 
@@ -86,19 +82,14 @@ fdbm_closed(obj)
     return Qfalse;
 }
 
-static VALUE fdbm_alloc _((VALUE));
 static VALUE
-fdbm_alloc(klass)
-    VALUE klass;
+fdbm_alloc(VALUE klass)
 {
     return Data_Wrap_Struct(klass, 0, free_dbm, 0);
 }
 
 static VALUE
-fdbm_initialize(argc, argv, obj)
-    int argc;
-    VALUE *argv;
-    VALUE obj;
+fdbm_initialize(int argc, VALUE *argv, VALUE obj)
 {
     VALUE file, vmode, vflags;
     DBM *dbm;
@@ -122,24 +113,24 @@ fdbm_initialize(argc, argv, obj)
 
     if (flags & RUBY_DBM_RW_BIT) {
         flags &= ~RUBY_DBM_RW_BIT;
-        dbm = dbm_open(RSTRING(file)->ptr, flags, mode);
+        dbm = dbm_open(RSTRING_PTR(file), flags, mode);
     }
     else {
         dbm = 0;
         if (mode >= 0) {
-            dbm = dbm_open(RSTRING(file)->ptr, O_RDWR|O_CREAT, mode);
+            dbm = dbm_open(RSTRING_PTR(file), O_RDWR|O_CREAT, mode);
         }
         if (!dbm) {
-            dbm = dbm_open(RSTRING(file)->ptr, O_RDWR, 0);
+            dbm = dbm_open(RSTRING_PTR(file), O_RDWR, 0);
         }
         if (!dbm) {
-            dbm = dbm_open(RSTRING(file)->ptr, O_RDONLY, 0);
+            dbm = dbm_open(RSTRING_PTR(file), O_RDONLY, 0);
         }
     }
 
     if (!dbm) {
 	if (mode == -1) return Qnil;
-	rb_sys_fail(RSTRING(file)->ptr);
+	rb_sys_fail(RSTRING_PTR(file));
     }
 
     dbmp = ALLOC(struct dbmdata);
@@ -151,10 +142,7 @@ fdbm_initialize(argc, argv, obj)
 }
 
 static VALUE
-fdbm_s_open(argc, argv, klass)
-    int argc;
-    VALUE *argv;
-    VALUE klass;
+fdbm_s_open(int argc, VALUE *argv, VALUE klass)
 {
     VALUE obj = Data_Wrap_Struct(klass, 0, free_dbm, 0);
 
@@ -170,16 +158,15 @@ fdbm_s_open(argc, argv, klass)
 }
 
 static VALUE
-fdbm_fetch(obj, keystr, ifnone)
-    VALUE obj, keystr, ifnone;
+fdbm_fetch(VALUE obj, VALUE keystr, VALUE ifnone)
 {
     datum key, value;
     struct dbmdata *dbmp;
     DBM *dbm;
 
     StringValue(keystr);
-    key.dptr = RSTRING(keystr)->ptr;
-    key.dsize = RSTRING(keystr)->len;
+    key.dptr = RSTRING_PTR(keystr);
+    key.dsize = RSTRING_LEN(keystr);
 
     GetDBM2(obj, dbmp, dbm);
     value = dbm_fetch(dbm, key);
@@ -192,17 +179,13 @@ fdbm_fetch(obj, keystr, ifnone)
 }
 
 static VALUE
-fdbm_aref(obj, keystr)
-    VALUE obj, keystr;
+fdbm_aref(VALUE obj, VALUE keystr)
 {
     return fdbm_fetch(obj, keystr, Qnil);
 }
 
 static VALUE
-fdbm_fetch_m(argc, argv, obj)
-    int argc;
-    VALUE *argv;
-    VALUE obj;
+fdbm_fetch_m(int argc, VALUE *argv, VALUE obj)
 {
     VALUE keystr, valstr, ifnone;
 
@@ -215,22 +198,21 @@ fdbm_fetch_m(argc, argv, obj)
 }
 
 static VALUE
-fdbm_index(obj, valstr)
-    VALUE obj, valstr;
+fdbm_index(VALUE obj, VALUE valstr)
 {
     datum key, val;
     struct dbmdata *dbmp;
     DBM *dbm;
 
     StringValue(valstr);
-    val.dptr = RSTRING(valstr)->ptr;
-    val.dsize = RSTRING(valstr)->len;
+    val.dptr = RSTRING_PTR(valstr);
+    val.dsize = RSTRING_LEN(valstr);
 
     GetDBM2(obj, dbmp, dbm);
     for (key = dbm_firstkey(dbm); key.dptr; key = dbm_nextkey(dbm)) {
 	val = dbm_fetch(dbm, key);
-	if (val.dsize == RSTRING(valstr)->len &&
-	    memcmp(val.dptr, RSTRING(valstr)->ptr, val.dsize) == 0) {
+	if (val.dsize == RSTRING_LEN(valstr) &&
+	    memcmp(val.dptr, RSTRING_PTR(valstr), val.dsize) == 0) {
 	    return rb_tainted_str_new(key.dptr, key.dsize);
 	}
     }
@@ -238,68 +220,31 @@ fdbm_index(obj, valstr)
 }
 
 static VALUE
-fdbm_indexes(argc, argv, obj)
-    int argc;
-    VALUE *argv;
-    VALUE obj;
+fdbm_select(VALUE obj)
 {
-    VALUE new;
-    int i;
+    VALUE new = rb_ary_new();
+    datum key, val;
+    DBM *dbm;
+    struct dbmdata *dbmp;
 
-    new = rb_ary_new2(argc);
-    for (i=0; i<argc; i++) {
-	rb_ary_push(new, fdbm_fetch(obj, argv[i], Qnil));
+    GetDBM2(obj, dbmp, dbm);
+    for (key = dbm_firstkey(dbm); key.dptr; key = dbm_nextkey(dbm)) {
+	VALUE assoc, v;
+	val = dbm_fetch(dbm, key);
+	assoc = rb_assoc_new(rb_tainted_str_new(key.dptr, key.dsize),
+			     rb_tainted_str_new(val.dptr, val.dsize));
+	v = rb_yield(assoc);
+	if (RTEST(v)) {
+	    rb_ary_push(new, assoc);
+	}
+	GetDBM2(obj, dbmp, dbm);
     }
 
     return new;
 }
 
 static VALUE
-fdbm_select(argc, argv, obj)
-    int argc;
-    VALUE *argv;
-    VALUE obj;
-{
-    VALUE new = rb_ary_new2(argc);
-    int i;
-
-    if (rb_block_given_p()) {
-        datum key, val;
-        DBM *dbm;
-        struct dbmdata *dbmp;
-
-	if (argc > 0) {
-	    rb_raise(rb_eArgError, "wrong number arguments(%d for 0)", argc);
-	}
-        GetDBM2(obj, dbmp, dbm);
-        for (key = dbm_firstkey(dbm); key.dptr; key = dbm_nextkey(dbm)) {
-            VALUE assoc, v;
-            val = dbm_fetch(dbm, key);
-            assoc = rb_assoc_new(rb_tainted_str_new(key.dptr, key.dsize),
-                                 rb_tainted_str_new(val.dptr, val.dsize));
-	    v = rb_yield(assoc);
-	    if (RTEST(v)) {
-		rb_ary_push(new, assoc);
-	    }
-	    GetDBM2(obj, dbmp, dbm);
-	}
-    }
-    else {
-	rb_warn("DBM#select(index..) is deprecated; use DBM#values_at");
-
-        for (i=0; i<argc; i++) {
-            rb_ary_push(new, fdbm_fetch(obj, argv[i], Qnil));
-        }
-    }
-
-    return new;
-}
-
-static VALUE
-fdbm_values_at(argc, argv, obj)
-    int argc;
-    VALUE *argv;
-    VALUE obj;
+fdbm_values_at(int argc, VALUE *argv, VALUE obj)
 {
     VALUE new = rb_ary_new2(argc);
     int i;
@@ -312,16 +257,14 @@ fdbm_values_at(argc, argv, obj)
 }
 
 static void
-fdbm_modify(obj)
-    VALUE obj;
+fdbm_modify(VALUE obj)
 {
     rb_secure(4);
     if (OBJ_FROZEN(obj)) rb_error_frozen("DBM");
 }
 
 static VALUE
-fdbm_delete(obj, keystr)
-    VALUE obj, keystr;
+fdbm_delete(VALUE obj, VALUE keystr)
 {
     datum key, value;
     struct dbmdata *dbmp;
@@ -330,10 +273,11 @@ fdbm_delete(obj, keystr)
 
     fdbm_modify(obj);
     StringValue(keystr);
-    key.dptr = RSTRING(keystr)->ptr;
-    key.dsize = RSTRING(keystr)->len;
+    key.dptr = RSTRING_PTR(keystr);
+    key.dsize = RSTRING_LEN(keystr);
 
     GetDBM2(obj, dbmp, dbm);
+
     value = dbm_fetch(dbm, key);
     if (value.dptr == 0) {
 	if (rb_block_given_p()) return rb_yield(keystr);
@@ -354,8 +298,7 @@ fdbm_delete(obj, keystr)
 }
 
 static VALUE
-fdbm_shift(obj)
-    VALUE obj;
+fdbm_shift(VALUE obj)
 {
     datum key, val;
     struct dbmdata *dbmp;
@@ -377,8 +320,7 @@ fdbm_shift(obj)
 }
 
 static VALUE
-fdbm_delete_if(obj)
-    VALUE obj;
+fdbm_delete_if(VALUE obj)
 {
     datum key, val;
     struct dbmdata *dbmp;
@@ -402,24 +344,23 @@ fdbm_delete_if(obj)
 	GetDBM2(obj, dbmp, dbm);
     }
 
-    for (i = 0; i < RARRAY(ary)->len; i++) {
-	keystr = RARRAY(ary)->ptr[i];
+    for (i = 0; i < RARRAY_LEN(ary); i++) {
+	keystr = RARRAY_PTR(ary)[i];
 	StringValue(keystr);
-	key.dptr = RSTRING(keystr)->ptr;
-	key.dsize = RSTRING(keystr)->len;
+	key.dptr = RSTRING_PTR(keystr);
+	key.dsize = RSTRING_LEN(keystr);
 	if (dbm_delete(dbm, key)) {
 	    rb_raise(rb_eDBMError, "dbm_delete failed");
 	}
     }
     if (status) rb_jump_tag(status);
-    if (n > 0) dbmp->di_size = n - RARRAY(ary)->len;
+    if (n > 0) dbmp->di_size = n - RARRAY_LEN(ary);
 
     return obj;
 }
 
 static VALUE
-fdbm_clear(obj)
-    VALUE obj;
+fdbm_clear(VALUE obj)
 {
     datum key;
     struct dbmdata *dbmp;
@@ -439,8 +380,7 @@ fdbm_clear(obj)
 }
 
 static VALUE
-fdbm_invert(obj)
-    VALUE obj;
+fdbm_invert(VALUE obj)
 {
     datum key, val;
     struct dbmdata *dbmp;
@@ -458,49 +398,36 @@ fdbm_invert(obj)
     return hash;
 }
 
-static VALUE each_pair _((VALUE));
+static VALUE fdbm_store(VALUE,VALUE,VALUE);
 
 static VALUE
-each_pair(obj)
-    VALUE obj;
-{
-    return rb_funcall(obj, rb_intern("each_pair"), 0, 0);
-}
-
-static VALUE fdbm_store _((VALUE,VALUE,VALUE));
-
-static VALUE
-update_i(pair, dbm)
-    VALUE pair, dbm;
+update_i(VALUE pair, VALUE dbm)
 {
     Check_Type(pair, T_ARRAY);
-    if (RARRAY(pair)->len < 2) {
+    if (RARRAY_LEN(pair) < 2) {
 	rb_raise(rb_eArgError, "pair must be [key, value]");
     }
-    fdbm_store(dbm, RARRAY(pair)->ptr[0], RARRAY(pair)->ptr[1]);
+    fdbm_store(dbm, RARRAY_PTR(pair)[0], RARRAY_PTR(pair)[1]);
     return Qnil;
 }
 
 static VALUE
-fdbm_update(obj, other)
-    VALUE obj, other;
+fdbm_update(VALUE obj, VALUE other)
 {
-    rb_iterate(each_pair, other, update_i, obj);
+    rb_block_call(other, rb_intern("each_pair"), 0, 0, update_i, obj);
     return obj;
 }
 
 static VALUE
-fdbm_replace(obj, other)
-    VALUE obj, other;
+fdbm_replace(VALUE obj, VALUE other)
 {
     fdbm_clear(obj);
-    rb_iterate(each_pair, other, update_i, obj);
+    rb_block_call(other, rb_intern("each_pair"), 0, 0, update_i, obj);
     return obj;
 }
 
 static VALUE
-fdbm_store(obj, keystr, valstr)
-    VALUE obj, keystr, valstr;
+fdbm_store(VALUE obj, VALUE keystr, VALUE valstr)
 {
     datum key, val;
     struct dbmdata *dbmp;
@@ -510,11 +437,11 @@ fdbm_store(obj, keystr, valstr)
     keystr = rb_obj_as_string(keystr);
     valstr = rb_obj_as_string(valstr);
 
-    key.dptr = RSTRING(keystr)->ptr;
-    key.dsize = RSTRING(keystr)->len;
+    key.dptr = RSTRING_PTR(keystr);
+    key.dsize = RSTRING_LEN(keystr);
 
-    val.dptr = RSTRING(valstr)->ptr;
-    val.dsize = RSTRING(valstr)->len;
+    val.dptr = RSTRING_PTR(valstr);
+    val.dsize = RSTRING_LEN(valstr);
 
     GetDBM2(obj, dbmp, dbm);
     dbmp->di_size = -1;
@@ -530,8 +457,7 @@ fdbm_store(obj, keystr, valstr)
 }
 
 static VALUE
-fdbm_length(obj)
-    VALUE obj;
+fdbm_length(VALUE obj)
 {
     datum key;
     struct dbmdata *dbmp;
@@ -550,8 +476,7 @@ fdbm_length(obj)
 }
 
 static VALUE
-fdbm_empty_p(obj)
-    VALUE obj;
+fdbm_empty_p(VALUE obj)
 {
     datum key;
     struct dbmdata *dbmp;
@@ -574,8 +499,7 @@ fdbm_empty_p(obj)
 }
 
 static VALUE
-fdbm_each_value(obj)
-    VALUE obj;
+fdbm_each_value(VALUE obj)
 {
     datum key, val;
     struct dbmdata *dbmp;
@@ -591,8 +515,7 @@ fdbm_each_value(obj)
 }
 
 static VALUE
-fdbm_each_key(obj)
-    VALUE obj;
+fdbm_each_key(VALUE obj)
 {
     datum key;
     struct dbmdata *dbmp;
@@ -607,8 +530,7 @@ fdbm_each_key(obj)
 }
 
 static VALUE
-fdbm_each_pair(obj)
-    VALUE obj;
+fdbm_each_pair(VALUE obj)
 {
     datum key, val;
     DBM *dbm;
@@ -629,8 +551,7 @@ fdbm_each_pair(obj)
 }
 
 static VALUE
-fdbm_keys(obj)
-    VALUE obj;
+fdbm_keys(VALUE obj)
 {
     datum key;
     struct dbmdata *dbmp;
@@ -648,8 +569,7 @@ fdbm_keys(obj)
 }
 
 static VALUE
-fdbm_values(obj)
-    VALUE obj;
+fdbm_values(VALUE obj)
 {
     datum key, val;
     struct dbmdata *dbmp;
@@ -667,16 +587,15 @@ fdbm_values(obj)
 }
 
 static VALUE
-fdbm_has_key(obj, keystr)
-    VALUE obj, keystr;
+fdbm_has_key(VALUE obj, VALUE keystr)
 {
     datum key, val;
     struct dbmdata *dbmp;
     DBM *dbm;
 
     StringValue(keystr);
-    key.dptr = RSTRING(keystr)->ptr;
-    key.dsize = RSTRING(keystr)->len;
+    key.dptr = RSTRING_PTR(keystr);
+    key.dsize = RSTRING_LEN(keystr);
 
     GetDBM2(obj, dbmp, dbm);
     val = dbm_fetch(dbm, key);
@@ -685,30 +604,28 @@ fdbm_has_key(obj, keystr)
 }
 
 static VALUE
-fdbm_has_value(obj, valstr)
-    VALUE obj, valstr;
+fdbm_has_value(VALUE obj, VALUE valstr)
 {
     datum key, val;
     struct dbmdata *dbmp;
     DBM *dbm;
 
     StringValue(valstr);
-    val.dptr = RSTRING(valstr)->ptr;
-    val.dsize = RSTRING(valstr)->len;
+    val.dptr = RSTRING_PTR(valstr);
+    val.dsize = RSTRING_LEN(valstr);
 
     GetDBM2(obj, dbmp, dbm);
     for (key = dbm_firstkey(dbm); key.dptr; key = dbm_nextkey(dbm)) {
 	val = dbm_fetch(dbm, key);
-	if (val.dsize == RSTRING(valstr)->len &&
-	    memcmp(val.dptr, RSTRING(valstr)->ptr, val.dsize) == 0)
+	if (val.dsize == RSTRING_LEN(valstr) &&
+	    memcmp(val.dptr, RSTRING_PTR(valstr), val.dsize) == 0)
 	    return Qtrue;
     }
     return Qfalse;
 }
 
 static VALUE
-fdbm_to_a(obj)
-    VALUE obj;
+fdbm_to_a(VALUE obj)
 {
     datum key, val;
     struct dbmdata *dbmp;
@@ -727,8 +644,7 @@ fdbm_to_a(obj)
 }
 
 static VALUE
-fdbm_to_hash(obj)
-    VALUE obj;
+fdbm_to_hash(VALUE obj)
 {
     datum key, val;
     struct dbmdata *dbmp;
@@ -747,14 +663,13 @@ fdbm_to_hash(obj)
 }
 
 static VALUE
-fdbm_reject(obj)
-    VALUE obj;
+fdbm_reject(VALUE obj)
 {
     return rb_hash_delete_if(fdbm_to_hash(obj));
 }
 
 void
-Init_dbm()
+Init_dbm(void)
 {
     rb_cDBM = rb_define_class("DBM", rb_cObject);
     rb_eDBMError = rb_define_class("DBMError", rb_eStandardError);
@@ -771,9 +686,7 @@ Init_dbm()
     rb_define_method(rb_cDBM, "[]=", fdbm_store, 2);
     rb_define_method(rb_cDBM, "store", fdbm_store, 2);
     rb_define_method(rb_cDBM, "index",  fdbm_index, 1);
-    rb_define_method(rb_cDBM, "indexes",  fdbm_indexes, -1);
-    rb_define_method(rb_cDBM, "indices",  fdbm_indexes, -1);
-    rb_define_method(rb_cDBM, "select",  fdbm_select, -1);
+    rb_define_method(rb_cDBM, "select",  fdbm_select, 0);
     rb_define_method(rb_cDBM, "values_at", fdbm_values_at, -1);
     rb_define_method(rb_cDBM, "length", fdbm_length, 0);
     rb_define_method(rb_cDBM, "size", fdbm_length, 0);
@@ -812,5 +725,7 @@ Init_dbm()
 
 #ifdef DB_VERSION_STRING
     rb_define_const(rb_cDBM, "VERSION",  rb_str_new2(DB_VERSION_STRING));
+#else
+    rb_define_const(rb_cDBM, "VERSION",  rb_str_new2("unknown"));
 #endif
 }
