@@ -1,6 +1,5 @@
 #
 #               tk/canvas.rb - Tk canvas classes
-#                       $Date: 2007-02-13 08:01:19 +0900 (Tue, 13 Feb 2007) $
 #                       by Yukihiro Matsumoto <matz@caelum.co.jp>
 #
 require 'tk'
@@ -40,7 +39,7 @@ module TkCanvasItemConfig
   private :__item_pathname
 end
 
-class TkCanvas<TkWindow
+class Tk::Canvas<TkWindow
   include TkCanvasItemConfig
   include Tk::Scrollable
 
@@ -573,6 +572,10 @@ class TkCanvas<TkWindow
   end
 end
 
+#TkCanvas = Tk::Canvas unless Object.const_defined? :TkCanvas
+Tk.__set_toplevel_aliases__(:Tk, Tk::Canvas, :TkCanvas)
+
+
 class TkcItem<TkObject
   extend Tk
   include TkcTagAccess
@@ -658,8 +661,8 @@ class TkcItem<TkObject
   ########################################
 
   def initialize(parent, *args)
-    #unless parent.kind_of?(TkCanvas)
-    #  fail ArgumentError, "expect TkCanvas for 1st argument"
+    #unless parent.kind_of?(Tk::Canvas)
+    #  fail ArgumentError, "expect Tk::Canvas for 1st argument"
     #end
     @parent = @c = parent
     @path = parent.path
