@@ -284,7 +284,7 @@ class CGI
   # Standard internet newline sequence
   EOL = CR + LF
 
-  REVISION = '$Id: cgi.rb 12050 2007-03-12 17:55:03Z knu $' #:nodoc:
+  REVISION = '$Id: cgi.rb 15526 2008-02-17 14:08:27Z kazu $' #:nodoc:
 
   NEEDS_BINMODE = true if /WIN/ni.match(RUBY_PLATFORM) 
 
@@ -339,8 +339,8 @@ class CGI
   #   url_encoded_string = CGI::escape("'Stop!' said Fred")
   #      # => "%27Stop%21%27+said+Fred"
   def CGI::escape(string)
-    string.gsub(/([^ a-zA-Z0-9_.-]+)/n) do
-      '%' + $1.unpack('H2' * $1.size).join('%').upcase
+    string.gsub(/([^ a-zA-Z0-9_.-]+)/) do
+      '%' + $1.unpack('H2' * $1.bytesize).join('%').upcase
     end.tr(' ', '+')
   end
 
@@ -349,17 +349,24 @@ class CGI
   #   string = CGI::unescape("%27Stop%21%27+said+Fred")
   #      # => "'Stop!' said Fred"
   def CGI::unescape(string)
-    string.tr('+', ' ').gsub(/((?:%[0-9a-fA-F]{2})+)/n) do
-      [$1.delete('%')].pack('H*')
+    enc = string.encoding
+    string.tr('+', ' ').gsub(/((?:%[0-9a-fA-F]{2})+)/) do
+      [$1.delete('%')].pack('H*').force_encoding(enc)
     end
   end
 
+  TABLE_FOR_ESCAPE_HTML__ = {
+    '&' => '&amp;',
+    '"' => '&quot;',
+    '<' => '&lt;',
+    '>' => '&gt;',
+  }
 
   # Escape special characters in HTML, namely &\"<>
   #   CGI::escapeHTML('Usage: foo "bar" <baz>')
   #      # => "Usage: foo &quot;bar&quot; &lt;baz&gt;"
   def CGI::escapeHTML(string)
-    string.gsub(/&/n, '&amp;').gsub(/\"/n, '&quot;').gsub(/>/n, '&gt;').gsub(/</n, '&lt;')
+    string.gsub(/[&\"<>]/, TABLE_FOR_ESCAPE_HTML__)
   end
 
 
@@ -367,39 +374,37 @@ class CGI
   #   CGI::unescapeHTML("Usage: foo &quot;bar&quot; &lt;baz&gt;")
   #      # => "Usage: foo \"bar\" <baz>"
   def CGI::unescapeHTML(string)
-    string.gsub(/&(amp|quot|gt|lt|\#[0-9]+|\#x[0-9A-Fa-f]+);/n) do
+    enc = string.encoding
+    string.gsub(/&(amp|quot|gt|lt|\#[0-9]+|\#x[0-9A-Fa-f]+);/) do
       match = $1.dup
       case match
       when 'amp'                 then '&'
       when 'quot'                then '"'
       when 'gt'                  then '>'
       when 'lt'                  then '<'
-      when /\A#0*(\d+)\z/n       then
+      when /\A#0*(\d+)\z/        then
         if Integer($1) < 256
-          Integer($1).chr
+          Integer($1).chr.force_encoding(enc)
         else
-          if Integer($1) < 65536 and ($KCODE[0] == ?u or $KCODE[0] == ?U)
-            [Integer($1)].pack("U")
-          else
-            "&##{$1};"
-          end
+          "&##{$1};"
         end
-      when /\A#x([0-9a-f]+)\z/ni then
+      when /\A#x([0-9a-f]+)\z/i then
         if $1.hex < 256
-          $1.hex.chr
+          $1.hex.chr.force_encoding(enc)
         else
-          if $1.hex < 65536 and ($KCODE[0] == ?u or $KCODE[0] == ?U)
-            [$1.hex].pack("U")
-          else
-            "&#x#{$1};"
-          end
+          "&#x#{$1};"
         end
       else
         "&#{match};"
       end
     end
   end
-
+  def CGI::escape_html(str)
+    escapeHTML(str)
+  end
+  def CGI::unescape_html(str)
+    unescapeHTML(str)
+  end
 
   # Escape only the tags of certain HTML elements in +string+.
   #
@@ -445,7 +450,12 @@ class CGI
       string
     end
   end
-
+  def CGI::escape_element(str)
+    escapeElement(str)
+  end
+  def CGI::unescape_element(str)
+    unescapeElement(str)
+  end
 
   # Format a +Time+ object as a String using the format specified by RFC 1123.
   #
@@ -709,13 +719,13 @@ class CGI
       require "nkf"
       case options["charset"]
       when /iso-2022-jp/ni
-        content = NKF::nkf('-m0 -x -j', content)
+        content = NKF::nkf('-j -m0 -x', content)
         options["language"] = "ja" unless options.has_key?("language")
       when /euc-jp/ni
-        content = NKF::nkf('-m0 -x -e', content)
+        content = NKF::nkf('-e -m0 -x', content)
         options["language"] = "ja" unless options.has_key?("language")
       when /shift_jis/ni
-        content = NKF::nkf('-m0 -x -s', content)
+        content = NKF::nkf('-s -m0 -x', content)
         options["language"] = "ja" unless options.has_key?("language")
       end
     end
@@ -953,11 +963,11 @@ class CGI
     end
 
     # Get the cookies as a hash of cookie-name=>Cookie pairs.
-    attr_accessor("cookies")
+    attr_accessor :cookies
 
     # Get the parameters as a hash of name=>values pairs, where
     # values is an Array.
-    attr("params")
+    attr_reader :params
 
     # Set all the parameters.
     def params=(hash)
@@ -968,7 +978,7 @@ class CGI
     def read_multipart(boundary, content_length)
       params = Hash.new([])
       boundary = "--" + boundary
-      quoted_boundary = Regexp.quote(boundary, "n")
+      quoted_boundary = Regexp.quote(boundary)
       buf = ""
       bufsize = 10 * 1024
       boundary_end=""
@@ -986,22 +996,9 @@ class CGI
 
       loop do
         head = nil
-        if 10240 < content_length
-          require "tempfile"
-          body = Tempfile.new("CGI")
-        else
-          begin
-            require "stringio"
-            body = StringIO.new
-          rescue LoadError
-            require "tempfile"
-            body = Tempfile.new("CGI")
-          end
-        end
-        body.binmode if defined? body.binmode
+        body = MorphingBody.new
 
         until head and /#{quoted_boundary}(?:#{EOL}|--)/n.match(buf)
-
           if (not head) and /#{EOL}#{EOL}/n.match(buf)
             buf = buf.sub(/\A((?:.|\n)*?#{EOL})#{EOL}/n) do
               head = $1.dup
@@ -1032,13 +1029,13 @@ class CGI
           if "--" == $2
             content_length = -1
           end
-         boundary_end = $2.dup
+          boundary_end = $2.dup
           ""
         end
 
         body.rewind
 
-        /Content-Disposition:.* filename=(?:"((?:\\.|[^\"])*)"|([^;]*))/ni.match(head)
+        /Content-Disposition:.* filename=(?:"((?:\\.|[^\"\s])*)"|([^;\s]*))/ni.match(head)
 	filename = ($1 or $2 or "")
 	if /Mac/ni.match(env_table['HTTP_USER_AGENT']) and
 	    /Mozilla/ni.match(env_table['HTTP_USER_AGENT']) and
@@ -1046,7 +1043,7 @@ class CGI
 	  filename = CGI::unescape(filename)
 	end
         
-        /Content-Type: (.*)/ni.match(head)
+        /Content-Type: ([^\s]*)/ni.match(head)
         content_type = ($1 or "")
 
         (class << body; self; end).class_eval do
@@ -1055,8 +1052,8 @@ class CGI
           define_method(:content_type) {content_type.dup.taint}
         end
 
-        /Content-Disposition:.* name="?([^\";]*)"?/ni.match(head)
-        name = $1.dup
+        /Content-Disposition:.* name="?([^\";\s]*)"?/ni.match(head)
+        name = ($1 || "").dup
 
         if params.has_key?(name)
           params[name].push(body)
@@ -1097,6 +1094,59 @@ class CGI
     end
     private :read_from_cmdline
 
+    # A wrapper class to use a StringIO object as the body and switch
+    # to a TempFile when the passed threshold is passed.
+    class MorphingBody
+      begin
+        require "stringio"
+        @@small_buffer = lambda{StringIO.new}
+      rescue LoadError
+        require "tempfile"
+        @@small_buffer = lambda{
+          n = Tempfile.new("CGI")
+          n.binmode
+          n
+        }
+      end
+
+      def initialize(morph_threshold = 10240)
+        @threshold = morph_threshold
+        @body = @@small_buffer.call
+        @cur_size = 0
+        @morph_check = true
+      end
+
+      def print(data)
+        if @morph_check && (@cur_size + data.size > @threshold)
+          convert_body
+        end
+        @body.print data
+      end
+      def rewind
+        @body.rewind
+      end
+      def path
+        @body.path
+      end
+
+      # returns the true body object.
+      def extract
+        @body
+      end
+
+      private
+      def convert_body
+        new_body = TempFile.new("CGI")
+        new_body.binmode if defined? @body.binmode
+        new_body.binmode if defined? new_body.binmode
+
+        @body.rewind
+        new_body.print @body.read
+        @body = new_body
+        @morph_check = false
+      end
+    end
+
     # Initialize the data from the query.
     #
     # Handles multipart forms (in particular, forms that involve file uploads).
@@ -1134,35 +1184,13 @@ class CGI
       @multipart
     end
 
-    module Value    # :nodoc:
-      def set_params(params)
-        @params = params
-      end
-      def [](idx, *args)
-        if args.size == 0
-          warn "#{caller(1)[0]}:CAUTION! cgi['key'] == cgi.params['key'][0]; if want Array, use cgi.params['key']"
-          @params[idx]
-        else
-          super[idx,*args]
-        end
-      end
-      def first
-        warn "#{caller(1)[0]}:CAUTION! cgi['key'] == cgi.params['key'][0]; if want Array, use cgi.params['key']"
-        self
-      end
-      alias last first
-      def to_a
-        @params || [self]
-      end
-      alias to_ary to_a   	# to be rhs of multiple assignment
-    end
-
     # Get the value for the parameter with a given key.
     #
     # If the parameter has multiple values, only the first will be 
     # retrieved; use #params() to get the array of values.
     def [](key)
       params = @params[key]
+      return '' unless params
       value = params[0]
       if @multipart
         if value
@@ -1174,8 +1202,6 @@ class CGI
         end
       else
         str = if value then value.dup else "" end
-        str.extend(Value)
-        str.set_params(params)
         str
       end
     end
@@ -1257,7 +1283,7 @@ class CGI
             else
               '="' + CGI::escapeHTML(value) + '"'
             end
-          }.to_s + ">"
+          }.join + ">"
       END
       s.sub!(/\Z/, " +") << append if append
       s
@@ -1480,7 +1506,7 @@ class CGI
             value[value.size - 1]
           end
         end
-      }.to_s
+      }.join
     end
 
 
@@ -1560,7 +1586,7 @@ class CGI
       if @output_hidden
         body += @output_hidden.collect{|k,v|
           "<INPUT TYPE=\"HIDDEN\" NAME=\"#{k}\" VALUE=\"#{v}\">"
-        }.to_s
+        }.join
       end
       super(attributes){body}
     end
@@ -1858,7 +1884,7 @@ class CGI
               }
             end
           end
-        }.to_s
+        }.join
       }
 
     end
@@ -1935,7 +1961,7 @@ class CGI
             value[value.size - 1]
           end
         end
-      }.to_s
+      }.join
     end
 
     # Generate a reset button Input element, as a String.
@@ -2265,15 +2291,15 @@ class CGI
       Apache.request.setup_cgi_env
     end
 
+      (class << self; self; end).class_eval do
+        const_set(:CGI_PARAMS,  [1])
+        const_set(:CGI_COOKIES, [2])
+      end
+
     extend QueryExtension
     @multipart = false
-    if defined?(CGI_PARAMS)
-      warn "do not use CGI_PARAMS and CGI_COOKIES"
-      @params = CGI_PARAMS.dup
-      @cookies = CGI_COOKIES.dup
-    else
-      initialize_query()  # set @params, @cookies
-    end
+
+    initialize_query()  # set @params, @cookies
     @output_cookies = nil
     @output_hidden = nil
 
