@@ -1,8 +1,7 @@
 #
 #   irb/ruby-lex.rb - ruby lexcal analizer
 #   	$Release Version: 0.9.5$
-#   	$Revision: 11708 $
-#   	$Date: 2007-02-12 16:01:19 -0700 (Mon, 12 Feb 2007) $
+#   	$Revision: 15267 $
 #   	by Keiju ISHITSUKA(keiju@ruby-lang.org)
 #
 # --
@@ -15,7 +14,7 @@ require "irb/slex"
 require "irb/ruby-token"
 
 class RubyLex
-  @RCS_ID='-$Id: ruby-lex.rb 11708 2007-02-12 23:01:19Z shyouhei $-'
+  @RCS_ID='-$Id: ruby-lex.rb 15267 2008-01-27 07:43:31Z naruse $-'
 
   extend Exception2MessageMapper
   def_exception(:AlreadyDefinedToken, "Already defined token(%s)")
@@ -104,6 +103,7 @@ class RubyLex
       @rests.push nil unless buf_input
     end
     c = @rests.shift
+    return if c == nil
     if @here_header
       @here_readed.push c
     else
@@ -188,7 +188,7 @@ class RubyLex
     prompt
     line = @input.call
     return nil unless line
-    @rests.concat line.split(//)
+    @rests.concat line.chars.to_a
     true
   end
   private :buf_input
@@ -234,7 +234,6 @@ class RubyLex
 	  unless l = lex
 	    throw :TERM_INPUT if @line == ''
 	  else
-	    #p l
 	    @line.concat l
 	    if @ltype or @continue or @indent > 0
 	      next
@@ -967,7 +966,7 @@ class RubyLex
       when /[0-7]/
 	match = /[0-7_]/
       when /[89]/
-	RubyLex.fail SyntaxError, "Illegal octal digit"
+	RubyLex.fail SyntaxError, "Invalid octal digit"
       else 
 	return Token(TkINTEGER)
       end
@@ -1054,6 +1053,12 @@ class RubyLex
 	  break
 	elsif @ltype != "'" && @ltype != "]" && @ltype != ":" and ch == "#"
 	  subtype = true
+	elsif ch == '\\' and @ltype == "'" #'
+	  case ch = getc
+	  when "\\", "\n", "'"
+	  else
+	    ungetc
+	  end
 	elsif ch == '\\' #'
 	  read_escape
 	end
