@@ -1,8 +1,8 @@
 #
 # = net/http.rb
 #
-# Copyright (c) 1999-2005 Yukihiro Matsumoto
-# Copyright (c) 1999-2005 Minero Aoki
+# Copyright (c) 1999-2007 Yukihiro Matsumoto
+# Copyright (c) 1999-2007 Minero Aoki
 # Copyright (c) 2001 GOTOU Yuuzou
 # 
 # Written and maintained by Minero Aoki <aamine@loveruby.net>.
@@ -22,7 +22,7 @@
 # http://www.ruby-lang.org/ja/man/?cmd=view;name=net%2Fhttp.rb
 # 
 #--
-# $Id: http.rb,v 1.100.2.10 2005/09/13 16:27:01 aamine Exp $
+# $Id: http.rb 15442 2008-02-12 06:18:06Z naruse $
 #++ 
 
 require 'net/protocol'
@@ -86,26 +86,32 @@ module Net   #:nodoc:
   #
   #     #1: Simple POST
   #     res = Net::HTTP.post_form(URI.parse('http://www.example.com/search.cgi'),
-  #                               {'q'=>'ruby', 'max'=>'50'})
+  #                               {'q' => 'ruby', 'max' => '50'})
   #     puts res.body
   #
   #     #2: POST with basic authentication
   #     res = Net::HTTP.post_form(URI.parse('http://jack:pass@www.example.com/todo.cgi'),
-  #                                         {'from'=>'2005-01-01', 'to'=>'2005-03-31'})
+  #                                         {'from' => '2005-01-01',
+  #                                          'to' => '2005-03-31'})
   #     puts res.body
   #
   #     #3: Detailed control
   #     url = URI.parse('http://www.example.com/todo.cgi')
   #     req = Net::HTTP::Post.new(url.path)
   #     req.basic_auth 'jack', 'pass'
-  #     req.set_form_data({'from'=>'2005-01-01', 'to'=>'2005-03-31'}, ';')
-  #     res = Net::HTTP.new(url.host, url.port).start { http.request(req) }
+  #     req.set_form_data({'from' => '2005-01-01', 'to' => '2005-03-31'}, ';')
+  #     res = Net::HTTP.new(url.host, url.port).start {|http| http.request(req) }
   #     case res
   #     when Net::HTTPSuccess, Net::HTTPRedirection
   #       # OK
   #     else
   #       res.error!
   #     end
+  #
+  #     #4: Multiple values
+  #     res = Net::HTTP.post_form(URI.parse('http://www.example.com/search.cgi'),
+  #                               {'q' => ['ruby', 'perl'], 'max' => '50'})
+  #     puts res.body
   # 
   # === Accessing via Proxy
   # 
@@ -278,9 +284,16 @@ module Net   #:nodoc:
   class HTTP < Protocol
 
     # :stopdoc:
-    Revision = %q$Revision: 1.100.2.10 $.split[1]
+    Revision = %q$Revision: 15442 $.split[1]
     HTTPVersion = '1.1'
-    @newimpl = true    # for backward compatability
+    @newimpl = true
+    begin
+      require 'zlib'
+      require 'stringio'  #for our purposes (unpacking gzip) lump these together
+      HAVE_ZLIB=true
+    rescue LoadError
+      HAVE_ZLIB=false
+    end
     # :startdoc:
 
     # Turns on net/http 1.2 (ruby 1.8) features.
@@ -461,7 +474,7 @@ module Net   #:nodoc:
       @address = address
       @port    = (port || HTTP.default_port)
       @curr_http_version = HTTPVersion
-      @seems_1_0_server = false
+      @no_keepalive_server = false
       @close_on_empty_response = false
       @socket  = nil
       @started = false
@@ -470,6 +483,9 @@ module Net   #:nodoc:
       @debug_output = nil
       @use_ssl = false
       @ssl_context = nil
+      @enable_post_connection_check = true
+      @compression = nil
+      @sspi_enabled = false
     end
 
     def inspect
@@ -523,11 +539,9 @@ module Net   #:nodoc:
 
     # returns true if use SSL/TLS with HTTP.
     def use_ssl?
-      @use_ssl
+      false   # redefined in net/https
     end
 
-    alias use_ssl use_ssl?   #:nodoc: obsolete
-
     # Opens TCP connection and HTTP session.
     # 
     # When this method is called with block, gives a HTTP object
@@ -562,10 +576,14 @@ module Net   #:nodoc:
       s = timeout(@open_timeout) { TCPSocket.open(conn_address(), conn_port()) }
       D "opened"
       if use_ssl?
-        unless @ssl_context.verify_mode
-          warn "warning: peer certificate won't be verified in this SSL session"
-          @ssl_context.verify_mode = OpenSSL::SSL::VERIFY_NONE
+        ssl_parameters = Hash.new
+        SSL_ATTRIBUTES.each do |name|
+          if value = instance_variable_get("@#{name}")
+            ssl_parameters[name] = value
+          end
         end
+        @ssl_context = OpenSSL::SSL::SSLContext.new
+        @ssl_context.set_params(ssl_parameters)
         s = OpenSSL::SSL::SSLSocket.new(s, @ssl_context)
         s.sync_close = true
       end
@@ -586,6 +604,9 @@ module Net   #:nodoc:
           HTTPResponse.read_new(@socket).value
         end
         s.connect
+        if @ssl_context.verify_mode != OpenSSL::SSL::VERIFY_NONE
+          s.post_connection_check(@address)
+        end
       end
       on_connect
     end
@@ -732,7 +753,18 @@ module Net   #:nodoc:
     public
 
     # Gets data from +path+ on the connected-to host.
-    # +header+ must be a Hash like { 'Accept' => '*/*', ... }.
+    # +initheader+ must be a Hash like { 'Accept' => '*/*', ... },
+    # and it defaults to an empty hash.
+    # If +initheader+ doesn't have the key 'accept-encoding', then
+    # a value of "gzip;q=1.0,deflate;q=0.6,identity;q=0.3" is used,
+    # so that gzip compression is used in preference to deflate 
+    # compression, which is used in preference to no compression. 
+    # Ruby doesn't have libraries to support the compress (Lempel-Ziv)
+    # compression, so that is not supported.  The intent of this is
+    # to reduce bandwidth by default.   If this routine sets up
+    # compression, then it does the decompression also, removing
+    # the header as well to prevent confusion.  Otherwise
+    # it leaves the body as it found it. 
     #
     # In version 1.1 (ruby 1.6), this method returns a pair of objects,
     # a Net::HTTPResponse object and the entity body string.
@@ -766,10 +798,33 @@ module Net   #:nodoc:
     #       end
     #     }
     #
-    def get(path, initheader = nil, dest = nil, &block) # :yield: +body_segment+
+    def get(path, initheader = {}, dest = nil, &block) # :yield: +body_segment+
       res = nil
+      if HAVE_ZLIB
+        unless  initheader.keys.any?{|k| k.downcase == "accept-encoding"}
+          initheader["accept-encoding"] = "gzip;q=1.0,deflate;q=0.6,identity;q=0.3"
+          @compression = true
+        end
+      end
       request(Get.new(path, initheader)) {|r|
-        r.read_body dest, &block
+        if r.key?("content-encoding") and @compression
+          @compression = nil # Clear it till next set.
+          the_body = r.read_body dest, &block
+          case r["content-encoding"]
+          when "gzip"
+            r.body= Zlib::GzipReader.new(StringIO.new(the_body)).read
+            r.delete("content-encoding")
+          when "deflate"
+            r.body= Zlib::Inflate.inflate(the_body);
+            r.delete("content-encoding")
+          when "identity"
+            ; # nothing needed
+          else 
+            ; # Don't do anything dramatic, unless we need to later
+          end
+        else
+          r.read_body dest, &block
+        end
         res = r
       }
       unless @newimpl
@@ -835,6 +890,10 @@ module Net   #:nodoc:
     #       end
     #     }
     #
+    # You should set Content-Type: header field for POST.
+    # If no Content-Type: field given, this method uses
+    # "application/x-www-form-urlencoded" by default.
+    #
     def post(path, data, initheader = nil, dest = nil, &block) # :yield: +body_segment+
       res = nil
       request(Post.new(path, initheader), data) {|r|
@@ -845,7 +904,6 @@ module Net   #:nodoc:
         res.value
         return res, res.body
       end
-
       res
     end
 
@@ -1034,34 +1092,34 @@ module Net   #:nodoc:
         }
       end
       if proxy_user()
-        unless use_ssl?
-          req.proxy_basic_auth proxy_user(), proxy_pass()
-        end
+        req.proxy_basic_auth proxy_user(), proxy_pass() unless use_ssl?
       end
-
       req.set_body_internal body
-      begin_transport req
-        req.exec @socket, @curr_http_version, edit_path(req.path)
-        begin
-          res = HTTPResponse.read_new(@socket)
-        end while res.kind_of?(HTTPContinue)
-        res.reading_body(@socket, req.response_body_permitted?) {
-          yield res if block_given?
-        }
-      end_transport req, res
-
+      res = transport_request(req, &block)
+      if sspi_auth?(res)
+        sspi_auth(req)
+        res = transport_request(req, &block)
+      end
       res
     end
 
     private
 
+    def transport_request(req)
+      begin_transport req
+      req.exec @socket, @curr_http_version, edit_path(req.path)
+      begin
+        res = HTTPResponse.read_new(@socket)
+      end while res.kind_of?(HTTPContinue)
+      res.reading_body(@socket, req.response_body_permitted?) {
+        yield res if block_given?
+      }
+      end_transport req, res
+      res
+    end
+
     def begin_transport(req)
-      if @socket.closed?
-        connect
-      end
-      if @seems_1_0_server
-        req['connection'] ||= 'close'
-      end
+      connect if @socket.closed?
       if not req.response_body_permitted? and @close_on_empty_response
         req['connection'] ||= 'close'
       end
@@ -1070,15 +1128,13 @@ module Net   #:nodoc:
 
     def end_transport(req, res)
       @curr_http_version = res.http_version
-      if not res.body and @close_on_empty_response
+      if @socket.closed?
+        D 'Conn socket closed'
+      elsif not res.body and @close_on_empty_response
         D 'Conn close'
         @socket.close
       elsif keep_alive?(req, res)
         D 'Conn keep-alive'
-        if @socket.closed?
-          D 'Conn (but seems 1.0 server)'
-          @seems_1_0_server = true
-        end
       else
         D 'Conn close'
         @socket.close
@@ -1086,13 +1142,40 @@ module Net   #:nodoc:
     end
 
     def keep_alive?(req, res)
-      return false if /close/i =~ req['connection'].to_s
-      return false if @seems_1_0_server
-      return true  if /keep-alive/i =~ res['connection'].to_s
-      return false if /close/i      =~ res['connection'].to_s
-      return true  if /keep-alive/i =~ res['proxy-connection'].to_s
-      return false if /close/i      =~ res['proxy-connection'].to_s
-      (@curr_http_version == '1.1')
+      return false if req.connection_close?
+      if @curr_http_version <= '1.0'
+        res.connection_keep_alive?
+      else   # HTTP/1.1 or later
+        not res.connection_close?
+      end
+    end
+
+    def sspi_auth?(res)
+      return false unless @sspi_enabled
+      if res.kind_of?(HTTPProxyAuthenticationRequired) and
+          proxy? and res["Proxy-Authenticate"].include?("Negotiate")
+        begin
+          require 'win32/sspi'
+          true
+        rescue LoadError
+          false
+        end
+      else
+        false
+      end
+    end
+
+    def sspi_auth(req)
+      n = Win32::SSPI::NegotiateAuth.new
+      req["Proxy-Authorization"] = "Negotiate #{n.get_initial_token}"
+      # Some versions of ISA will close the connection if this isn't present.
+      req["Connection"] = "Keep-Alive"
+      req["Proxy-Connection"] = "Keep-Alive"
+      res = transport_request(req)
+      authphrase = res["Proxy-Authenticate"]  or return res
+      req["Proxy-Authorization"] = "Negotiate #{n.complete_authentication(authphrase)}"
+    rescue => err
+      raise HTTPAuthenticationError.new('HTTP authentication failed', err)
     end
 
     #
@@ -1158,7 +1241,7 @@ module Net   #:nodoc:
         @header.delete key.downcase
         return val
       end
-      @header[key.downcase] = Array(val).map {|s| s.to_str }
+      @header[key.downcase] = [val]
     end
 
     # [Ruby 1.8.3]
@@ -1178,9 +1261,9 @@ module Net   #:nodoc:
     #
     def add_field(key, val)
       if @header.key?(key.downcase)
-        @header[key.downcase].concat Array(val)
+        @header[key.downcase].push val
       else
-        @header[key.downcase] = Array(val).dup
+        @header[key.downcase] = [val]
       end
     end
 
@@ -1365,44 +1448,80 @@ module Net   #:nodoc:
       r.end - r.begin
     end
 
+    # Returns a content type string such as "text/html".
+    # This method returns nil if Content-Type: header field does not exist.
     def content_type
-      "#{main_type()}/#{sub_type()}"
+      return nil unless main_type()
+      if sub_type()
+      then "#{main_type()}/#{sub_type()}"
+      else main_type()
+      end
     end
 
+    # Returns a content type string such as "text".
+    # This method returns nil if Content-Type: header field does not exist.
     def main_type
       return nil unless @header['content-type']
       self['Content-Type'].split(';').first.to_s.split('/')[0].to_s.strip
     end
     
+    # Returns a content type string such as "html".
+    # This method returns nil if Content-Type: header field does not exist
+    # or sub-type is not given (e.g. "Content-Type: text").
     def sub_type
       return nil unless @header['content-type']
-      self['Content-Type'].split(';').first.to_s.split('/')[1].to_s.strip
+      main, sub = *self['Content-Type'].split(';').first.to_s.split('/')
+      return nil unless sub
+      sub.strip
     end
 
+    # Returns content type parameters as a Hash as like
+    # {"charset" => "iso-2022-jp"}.
     def type_params
       result = {}
-      self['Content-Type'].to_s.split(';')[1..-1].each do |param|
+      list = self['Content-Type'].to_s.split(';')
+      list.shift
+      list.each do |param|
         k, v = *param.split('=', 2)
         result[k.strip] = v.strip
       end
       result
     end
 
+    # Set Content-Type: header field by +type+ and +params+.
+    # +type+ must be a String, +params+ must be a Hash.
     def set_content_type(type, params = {})
       @header['content-type'] = [type + params.map{|k,v|"; #{k}=#{v}"}.join('')]
     end
 
     alias content_type= set_content_type
 
+    # Set header fields and a body from HTML form data.
+    # +params+ should be a Hash containing HTML form data.
+    # Optional argument +sep+ means data record separator.
+    #
+    # This method also set Content-Type: header field to
+    # application/x-www-form-urlencoded.
+    #
+    # Example:
+    #    http.form_data = {"q" => "ruby", "lang" => "en"}
+    #    http.form_data = {"q" => ["ruby", "perl"], "lang" => "en"}
+    #    http.set_form_data({"q" => "ruby", "lang" => "en"}, ';')
+    #    
     def set_form_data(params, sep = '&')
-      self.body = params.map {|k,v| "#{urlencode(k.to_s)}=#{urlencode(v.to_s)}" }.join(sep)
+      self.body = params.map {|k, v| encode_kvpair(k, v) }.flatten.join(sep)
       self.content_type = 'application/x-www-form-urlencoded'
     end
 
     alias form_data= set_form_data
 
+    def encode_kvpair(k, vs)
+      Array(vs).map {|v| "#{urlencode(k)}=#{urlencode(v.to_s)}" }
+    end
+    private :encode_kvpair
+
     def urlencode(str)
-      str.gsub(/[^a-zA-Z0-9_\.\-]/n) {|s| sprintf('%%%02x', s[0]) }
+      str.gsub(/[^a-zA-Z0-9_\.\-]/n) { sprintf('%%%02x', $&[0]) }
     end
     private :urlencode
 
@@ -1421,6 +1540,24 @@ module Net   #:nodoc:
     end
     private :basic_encode
 
+    def connection_close?
+      tokens(@header['connection']).include?('close') or
+      tokens(@header['proxy-connection']).include?('close')
+    end
+
+    def connection_keep_alive?
+      tokens(@header['connection']).include?('keep-alive') or
+      tokens(@header['proxy-connection']).include?('keep-alive')
+    end
+
+    def tokens(vals)
+      return [] unless vals
+      vals.map {|v| v.split(',') }.flatten\
+          .reject {|str| str.strip.empty? }\
+          .map {|tok| tok.strip.downcase }
+    end
+    private :tokens
+
   end
 
 
@@ -1438,10 +1575,12 @@ module Net   #:nodoc:
       @method = m
       @request_has_body = reqbody
       @response_has_body = resbody
+      raise ArgumentError, "no HTTP request path given" unless path
       raise ArgumentError, "HTTP request path is empty" if path.empty?
       @path = path
       initialize_http_header initheader
       self['Accept'] ||= '*/*'
+      self['User-Agent'] ||= 'Ruby'
       @body = nil
       @body_stream = nil
     end
@@ -1506,20 +1645,17 @@ module Net   #:nodoc:
     def send_request_with_body(sock, ver, path, body)
       self.content_length = body.length
       delete 'Transfer-Encoding'
-      unless content_type()
-        warn 'net/http: warning: Content-Type did not set; using application/x-www-form-urlencoded' if $VERBOSE
-        set_content_type 'application/x-www-form-urlencoded'
-      end
+      supply_default_content_type
       write_header sock, ver, path
       sock.write body
     end
 
     def send_request_with_body_stream(sock, ver, path, f)
-      raise ArgumentError, "Content-Length not given and Transfer-Encoding is not `chunked'" unless content_length() or chunked?
-      unless content_type()
-        warn 'net/http: warning: Content-Type did not set; using application/x-www-form-urlencoded' if $VERBOSE
-        set_content_type 'application/x-www-form-urlencoded'
+      unless content_length() or chunked?
+        raise ArgumentError,
+            "Content-Length not given and Transfer-Encoding is not `chunked'"
       end
+      supply_default_content_type
       write_header sock, ver, path
       if chunked?
         while s = f.read(1024)
@@ -1533,6 +1669,12 @@ module Net   #:nodoc:
       end
     end
 
+    def supply_default_content_type
+      return if content_type()
+      warn 'net/http: warning: Content-Type did not set; using application/x-www-form-urlencoded' if $VERBOSE
+      set_content_type 'application/x-www-form-urlencoded'
+    end
+
     def write_header(sock, ver, path)
       buf = "#{@method} #{path} HTTP/#{ver}\r\n"
       each_capitalized do |k,v|
@@ -1964,7 +2106,7 @@ module Net   #:nodoc:
       '416' => HTTPRequestedRangeNotSatisfiable,
       '417' => HTTPExpectationFailed,
 
-      '501' => HTTPInternalServerError,
+      '500' => HTTPInternalServerError,
       '501' => HTTPNotImplemented,
       '502' => HTTPBadGateway,
       '503' => HTTPServiceUnavailable,
@@ -2166,6 +2308,12 @@ module Net   #:nodoc:
       read_body()
     end
 
+    # Because it may be necessary to modify the body, Eg, decompression
+    # this method facilitates that.
+    def body=(value)
+      @body = value
+    end
+
     alias entity body   #:nodoc: obsolete
 
     private
