@@ -203,7 +203,7 @@
 #
 class OptionParser
   # :stopdoc:
-  RCSID = %w$Id: optparse.rb 16923 2008-06-07 18:07:29Z shyouhei $[1..-1].each {|s| s.freeze}.freeze
+  RCSID = %w$Id: optparse.rb 13823 2007-11-04 20:17:06Z nobu $[1..-1].each {|s| s.freeze}.freeze
   Version = (RCSID[1].split('.').collect {|s| s.to_i}.extend(Comparable).freeze if RCSID[1])
   LastModified = (Time.gm(*RCSID[2, 2].join('-').scan(/\d+/).collect {|s| s.to_i}) if RCSID[2])
   Release = RCSID[2]
@@ -221,7 +221,7 @@ class OptionParser
     def complete(key, icase = false, pat = nil)
       pat ||= Regexp.new('\A' + Regexp.quote(key).gsub(/\w+\b/, '\&\w*'),
                          icase)
-      canon, sw, k, v, cn = nil
+      canon, sw, cn = nil
       candidates = []
       each do |k, *v|
         (if Regexp === k
@@ -321,10 +321,10 @@ class OptionParser
     # argument pattern. Yields when the pattern doesn't match substring.
     #
     def parse_arg(arg)
-      pattern or return nil, arg
+      pattern or return nil, [arg]
       unless m = pattern.match(arg)
         yield(InvalidArgument, arg)
-        return arg, nil
+        return arg, []
       end
       if String === m
         m = [s = m]
@@ -345,11 +345,11 @@ class OptionParser
     # conversion. Yields at semi-error condition instead of raising an
     # exception.
     #
-    def conv_arg(arg, val = nil)
+    def conv_arg(arg, val = [])
       if conv
         val = conv.call(*val)
       else
-        val = proc {|val| val}.call(*val)
+        val = proc {|v| v}.call(*val)
       end
       return arg, block, val
     end
@@ -368,7 +368,7 @@ class OptionParser
     # +indent+:: Prefix string indents all summarized lines.
     #
     def summarize(sdone = [], ldone = [], width = 1, max = width - 1, indent = "")
-      sopts, lopts, s = [], [], nil
+      sopts, lopts = [], [], nil
       @short.each {|s| sdone.fetch(s) {sopts << s}; sdone[s] = true} if @short
       @long.each {|s| ldone.fetch(s) {lopts << s}; ldone[s] = true} if @long
       return if sopts.empty? and lopts.empty? # completely hidden
@@ -384,9 +384,9 @@ class OptionParser
       end
 
       left[0] << arg if arg
-      mlen = left.collect {|s| s.length}.max.to_i
+      mlen = left.collect {|ss| ss.length}.max.to_i
       while mlen > width and l = left.shift
-        mlen = left.collect {|s| s.length}.max.to_i if l.length == mlen
+        mlen = left.collect {|ss| ss.length}.max.to_i if l.length == mlen
         yield(indent + l)
       end
 
@@ -406,7 +406,7 @@ class OptionParser
       to
     end
 
-    def match_nonswitch?(str) # :nodoc:
+    def match_nonswitch?(str)  # :nodoc:
       @pattern =~ str unless @short or @long
     end
 
@@ -451,7 +451,7 @@ class OptionParser
           raise MissingArgument if argv.empty?
           arg = argv.shift
         end
-        conv_arg(*parse_arg(arg) {|*exc| raise(*exc)})
+        conv_arg(*parse_arg(arg, &method(:raise)))
       end
     end
 
@@ -555,7 +555,6 @@ class OptionParser
     # +nlopts+:: Negated long style options list.
     #
     def update(sw, sopts, lopts, nsw = nil, nlopts = nil)
-      o = nil
       sopts.each {|o| @short[o] = sw} if sopts
       lopts.each {|o| @long[o] = sw} if lopts
       nlopts.each {|o| @long[o] = nsw} if nsw and nlopts
@@ -633,8 +632,10 @@ class OptionParser
       list.each do |opt|
         if opt.respond_to?(:summarize) # perhaps OptionParser::Switch
           opt.summarize(*args, &block)
-        elsif !opt or opt.empty?
+        elsif !opt
           yield("")
+        elsif opt.respond_to?(:each_line)
+          opt.each_line(&block)
         else
           opt.each(&block)
         end
@@ -661,9 +662,10 @@ class OptionParser
     # Completion for hash key.
     #
     def match(key)
-      return key, *fetch(key) {
+      *values = fetch(key) {
         raise AmbiguousArgument, catch(:ambiguous) {return complete(key)}
       }
+      return key, *values
     end
   end
 
@@ -1060,7 +1062,6 @@ class OptionParser
     default_style = Switch::NoArgument
     default_pattern = nil
     klass = nil
-    o = nil
     n, q, a = nil
 
     opts.each do |o|
@@ -1077,7 +1078,7 @@ class OptionParser
       # directly specified pattern(any object possible to match)
       if !(String === o) and o.respond_to?(:match)
         pattern = notwice(o, pattern, 'pattern')
-        conv = (pattern.method(:convert).to_proc if pattern.respond_to?(:convert))
+        conv = pattern.method(:convert).to_proc if pattern.respond_to?(:convert)
         next
       end
 
@@ -1090,11 +1091,11 @@ class OptionParser
         when CompletingHash
         when nil
           pattern = CompletingHash.new
-          conv = (pattern.method(:convert).to_proc if pattern.respond_to?(:convert))
+          conv = pattern.method(:convert).to_proc if pattern.respond_to?(:convert)
         else
           raise ArgumentError, "argument pattern given twice"
         end
-        o.each {|(o, *v)| pattern[o] = v.fetch(0) {o}}
+        o.each {|pat, *v| pattern[pat] = v.fetch(0) {pat}}
       when Module
         raise ArgumentError, "unsupported argument type: #{o}"
       when *ArgumentStyle.keys
@@ -1242,8 +1243,8 @@ class OptionParser
   end
 
   def parse_in_order(argv = default_argv, setter = nil, &nonopt)  # :nodoc:
-    opt, arg, sw, val, rest = nil
-    nonopt ||= proc {|arg| throw :terminate, arg}
+    opt, arg, val, rest = nil
+    nonopt ||= proc {|a| throw :terminate, a}
     argv.unshift(arg) if arg = catch(:terminate) {
       while arg = argv.shift
         case arg
@@ -1297,7 +1298,8 @@ class OptionParser
         # non-option argument
         else
           catch(:prune) do
-            visit(:each_option) do |sw|
+            visit(:each_option) do |sw0|
+              sw = sw0
               sw.block.call(arg) if Switch === sw and sw.match_nonswitch?(arg)
             end
             nonopt.call(arg)
@@ -1328,8 +1330,7 @@ class OptionParser
   #
   def permute!(argv = default_argv)
     nonopts = []
-    arg = nil
-    order!(argv) {|arg| nonopts << arg}
+    order!(argv, &nonopts.method(:<<))
     argv[0, 0] = nonopts
     argv
   end
@@ -1406,7 +1407,6 @@ class OptionParser
   # +block+.
   #
   def visit(id, *args, &block)
-    el = nil
     @stack.reverse_each do |el|
       el.send(id, *args, &block)
     end
@@ -1439,7 +1439,7 @@ class OptionParser
       search(typ, opt) {|sw| return [sw, opt]} # exact match or...
     end
     raise AmbiguousOption, catch(:ambiguous) {
-      visit(:complete, typ, opt, icase, *pat) {|opt, *sw| return sw}
+      visit(:complete, typ, opt, icase, *pat) {|o, *sw| return sw}
       raise InvalidOption, opt
     }
   end
@@ -1558,7 +1558,7 @@ class OptionParser
   #
   accept(Array) do |s,|
     if s
-      s = s.split(',').collect {|s| s unless s.empty?}
+      s = s.split(',').collect {|ss| ss unless ss.empty?}
     end
     s
   end
