@@ -13,7 +13,7 @@ module REXML
 		#  CData.new( "Here is some CDATA" )
 		#  CData.new( "Some unprocessed data", respect_whitespace_TF, parent_element )
 		def initialize( first, whitespace=true, parent=nil )
-			super( first, whitespace, parent, true, true, ILLEGAL )
+			super( first, whitespace, parent, false, true, ILLEGAL )
 		end
 
 		# Make a copy of this object
