@@ -296,7 +296,7 @@ module REXML
       raise "First argument must be either an element name, or an Element object" if element.nil?
       el = @elements.add(element)
       attrs.each do |key, value|
-        el.attributes[key]=Attribute.new(key,value,self)
+        el.attributes[key]=value
       end	if attrs.kind_of? Hash
       el
     end
@@ -552,9 +552,25 @@ module REXML
 
     def attribute( name, namespace=nil )
       prefix = nil
-      prefix = namespaces.index(namespace) if namespace
+      if namespaces.respond_to? :key
+        prefix = namespaces.key(namespace) if namespace
+      else
+        prefix = namespaces.index(namespace) if namespace
+      end
       prefix = nil if prefix == 'xmlns'
-      attributes.get_attribute( "#{prefix ? prefix + ':' : ''}#{name}" )
+
+      ret_val = 
+        attributes.get_attribute( "#{prefix ? prefix + ':' : ''}#{name}" )
+
+      return ret_val unless ret_val.nil?
+      return nil if prefix.nil?
+
+      # now check that prefix'es namespace is not the same as the
+      # default namespace
+      return nil unless ( namespaces[ prefix ] == namespaces[ 'xmlns' ] )
+
+      attributes.get_attribute( name )
+
     end
 
     # Evaluates to +true+ if this element has any attributes set, false
@@ -671,7 +687,7 @@ module REXML
     #  out = ''
     #  doc.write( out )     #-> doc is written to the string 'out'
     #  doc.write( $stdout ) #-> doc written to the console
-    def write(writer=$stdout, indent=-1, transitive=false, ie_hack=false)
+    def write(output=$stdout, indent=-1, transitive=false, ie_hack=false)
       Kernel.warn("#{self.class.name}.write is deprecated.  See REXML::Formatters")
       formatter = if indent > -1
           if transitive
@@ -704,7 +720,6 @@ module REXML
     # A private helper method
     def each_with_something( test, max=0, name=nil )
       num = 0
-      child=nil
       @elements.each( name ){ |child|
         yield child if test.call(child) and num += 1
         return if max>0 and num == max
@@ -754,7 +769,6 @@ module REXML
         raise "index (#{index}) must be >= 1" if index < 1
         name = literalize(name) if name
         num = 0
-        child = nil
         @element.find { |child|
           child.kind_of? Element and
           (name.nil? ? true : child.has_name?( name )) and 
@@ -1013,7 +1027,7 @@ module REXML
     #  doc.root.attributes.each {|name, value| p name+" => "+value }
     def each
       each_attribute do |attr|
-        yield attr.expanded_name, attr.value
+        yield [attr.expanded_name, attr.value]
       end
     end
 
@@ -1215,13 +1229,17 @@ module REXML
     # 
     # Method contributed by Henrik Martensson
     def get_attribute_ns(namespace, name)
+      result = nil
       each_attribute() { |attribute|
         if name == attribute.name &&
-          namespace == attribute.namespace()
-          return attribute
+          namespace == attribute.namespace() &&
+          ( !namespace.empty? || !attribute.fully_expanded_name.index(':') )
+          # foo will match xmlns:foo, but only if foo isn't also an attribute
+          result = attribute if !result or !namespace.empty? or 
+                                !attribute.fully_expanded_name.index(':')
         end
       }
-      nil
+      result
     end
   end
 end
