@@ -10,10 +10,6 @@ class TestRegexp < Test::Unit::TestCase
     $VERBOSE = @verbose
   end
 
-  def test_ruby_core_27247
-    assert_match(/(a){2}z/, "aaz")
-  end
-
   def test_ruby_dev_24643
     assert_nothing_raised("[ruby-dev:24643]") {
       /(?:(?:[a]*[a])?b)*a*$/ =~ "aabaaca"
@@ -33,6 +29,13 @@ class TestRegexp < Test::Unit::TestCase
     assert_match(Regexp.new(s, nil, "u"), "\xC3\xBE")
   end
 
+  def test_yoshidam_net_20041111_2
+    assert_raise(RegexpError) do
+      s = "[\xFF-\xFF]".force_encoding("utf-8")
+      Regexp.new(s, nil, "u")
+    end
+  end
+
   def test_ruby_dev_31309
     assert_equal('Ruby', 'Ruby'.sub(/[^a-z]/i, '-'))
   end
@@ -43,7 +46,83 @@ class TestRegexp < Test::Unit::TestCase
   end
 
   def test_to_s
-    assert_equal '(?-mix:\000)', Regexp.new("\0").to_s
+    assert_equal '(?-mix:\x00)', Regexp.new("\0").to_s
+  end
+
+  def test_union
+    assert_equal :ok, begin
+      Regexp.union(
+        "a",
+        Regexp.new("\xc2\xa1".force_encoding("euc-jp")),
+        Regexp.new("\xc2\xa1".force_encoding("utf-8")))
+      :ng
+    rescue ArgumentError
+      :ok
+    end
+  end
+
+  def test_named_capture
+    m = /&(?<foo>.*?);/.match("aaa &amp; yyy")
+    assert_equal("amp", m["foo"])
+    assert_equal("amp", m[:foo])
+    assert_equal(5, m.begin(:foo))
+    assert_equal(8, m.end(:foo))
+    assert_equal([5,8], m.offset(:foo))
+
+    assert_equal("aaa [amp] yyy",
+      "aaa &amp; yyy".sub(/&(?<foo>.*?);/, '[\k<foo>]'))
+
+    assert_equal('#<MatchData "&amp; y" foo:"amp">',
+      /&(?<foo>.*?); (y)/.match("aaa &amp; yyy").inspect)
+    assert_equal('#<MatchData "&amp; y" 1:"amp" 2:"y">',
+      /&(.*?); (y)/.match("aaa &amp; yyy").inspect)
+    assert_equal('#<MatchData "&amp; y" foo:"amp" bar:"y">',
+      /&(?<foo>.*?); (?<bar>y)/.match("aaa &amp; yyy").inspect)
+    assert_equal('#<MatchData "&amp; y" foo:"amp" foo:"y">',
+      /&(?<foo>.*?); (?<foo>y)/.match("aaa &amp; yyy").inspect)
+
+    /(?<id>[A-Za-z_]+)/ =~ "!abc"
+    assert_equal("abc", Regexp.last_match(:id))
+
+    /a/ =~ "b" # doesn't match.
+    assert_equal(nil, Regexp.last_match)
+    assert_equal(nil, Regexp.last_match(1))
+    assert_equal(nil, Regexp.last_match(:foo))
+
+    assert_equal(["foo", "bar"], /(?<foo>.)(?<bar>.)/.names)
+    assert_equal(["foo"], /(?<foo>.)(?<foo>.)/.names)
+    assert_equal([], /(.)(.)/.names)
+
+    assert_equal(["foo", "bar"], /(?<foo>.)(?<bar>.)/.match("ab").names)
+    assert_equal(["foo"], /(?<foo>.)(?<foo>.)/.match("ab").names)
+    assert_equal([], /(.)(.)/.match("ab").names)
+
+    assert_equal({"foo"=>[1], "bar"=>[2]},
+                 /(?<foo>.)(?<bar>.)/.named_captures)
+    assert_equal({"foo"=>[1, 2]},
+                 /(?<foo>.)(?<foo>.)/.named_captures)
+    assert_equal({}, /(.)(.)/.named_captures)
+
+    assert_equal("a[b]c", "abc".sub(/(?<x>[bc])/, "[\\k<x>]"))
+  end
+
+  def test_assign_named_capture
+    assert_equal("a", eval('/(?<foo>.)/ =~ "a"; foo'))
+    assert_equal("a", eval('foo = 1; /(?<foo>.)/ =~ "a"; foo'))
+    assert_equal("a", eval('1.times {|foo| /(?<foo>.)/ =~ "a"; break foo }'))
+    assert_nothing_raised { eval('/(?<Foo>.)/ =~ "a"') }
+    assert_nil(eval('/(?<Foo>.)/ =~ "a"; defined? Foo'))
+  end
+
+  def test_assign_named_capture_to_reserved_word
+    /(?<nil>.)/ =~ "a"
+    assert(!local_variables.include?(:nil), "[ruby-dev:32675]")
+  end
+
+  def test_match_regexp
+    r = /./
+    m = r.match("a")
+    assert_equal(r, m.regexp)
   end
 
   def test_source
@@ -54,12 +133,12 @@ class TestRegexp < Test::Unit::TestCase
     assert_equal('//', //.inspect)
     assert_equal('//i', //i.inspect)
     assert_equal('/\//i', /\//i.inspect)
-    assert_equal('/\//i', %r"#{'/'}"i.inspect)
+    assert_equal('/\//i', /#{'/'}/i.inspect)
     assert_equal('/\/x/i', /\/x/i.inspect)
-    assert_equal('/\000/i', /#{"\0"}/i.inspect)
+    assert_equal('/\x00/i', /#{"\0"}/i.inspect)
     assert_equal("/\n/i", /#{"\n"}/i.inspect)
     s = [0xff].pack("C")
-    assert_equal('/\/'+s.dump.delete('"')+'/i', /\/#{s}/i.inspect)
+    assert_equal('/\/'+s+'/i', /\/#{s}/i.inspect)
   end
 
   def test_char_to_option
@@ -101,10 +180,17 @@ class TestRegexp < Test::Unit::TestCase
 
   def test_match_init_copy
     m = /foo/.match("foo")
+    assert_equal(/foo/, m.dup.regexp)
     assert_raise(TypeError) do
       m.instance_eval { initialize_copy(nil) }
     end
     assert_equal([0, 3], m.offset(0))
+    assert_equal(/foo/, m.dup.regexp)
+  end
+
+  def test_match_regexp
+    re = /foo/
+    assert_equal(re, re.match("foo").regexp)
   end
 
   def test_match_size
@@ -112,6 +198,38 @@ class TestRegexp < Test::Unit::TestCase
     assert_equal(5, m.size)
   end
 
+  def test_match_offset_begin_end
+    m = /(?<x>b..)/.match("foobarbaz")
+    assert_equal([3, 6], m.offset("x"))
+    assert_equal(3, m.begin("x"))
+    assert_equal(6, m.end("x"))
+    assert_raise(IndexError) { m.offset("y") }
+    assert_raise(IndexError) { m.offset(2) }
+    assert_raise(IndexError) { m.begin(2) }
+    assert_raise(IndexError) { m.end(2) }
+
+    m = /(?<x>q..)?/.match("foobarbaz")
+    assert_equal([nil, nil], m.offset("x"))
+    assert_equal(nil, m.begin("x"))
+    assert_equal(nil, m.end("x"))
+
+    m = /\A\u3042(.)(.)?(.)\z/.match("\u3042\u3043\u3044")
+    assert_equal([1, 2], m.offset(1))
+    assert_equal([nil, nil], m.offset(2))
+    assert_equal([2, 3], m.offset(3))
+  end
+
+  def test_match_to_s
+    m = /(?<x>b..)/.match("foobarbaz")
+    assert_equal("bar", m.to_s)
+  end
+
+  def test_match_pre_post
+    m = /(?<x>b..)/.match("foobarbaz")
+    assert_equal("foo", m.pre_match)
+    assert_equal("baz", m.post_match)
+  end
+
   def test_match_array
     m = /(...)(...)(...)(...)?/.match("foobarbaz")
     assert_equal(["foobarbaz", "foo", "bar", "baz", nil], m.to_a)
@@ -127,6 +245,7 @@ class TestRegexp < Test::Unit::TestCase
     assert_equal("foo", m[1])
     assert_equal(["foo", "bar", "baz"], m[1..3])
     assert_nil(m[5])
+    assert_raise(IndexError) { m[:foo] }
   end
 
   def test_match_values_at
@@ -134,6 +253,11 @@ class TestRegexp < Test::Unit::TestCase
     assert_equal(["foo", "bar", "baz"], m.values_at(1, 2, 3))
   end
 
+  def test_match_string
+    m = /(?<x>b..)/.match("foobarbaz")
+    assert_equal("foobarbaz", m.string)
+  end
+
   def test_match_inspect
     m = /(...)(...)(...)(...)?/.match("foobarbaz")
     assert_equal('#<MatchData "foobarbaz" 1:"foo" 2:"bar" 3:"baz" 4:nil>', m.inspect)
@@ -151,18 +275,34 @@ class TestRegexp < Test::Unit::TestCase
       Thread.new { $SAFE = 4; re.instance_eval { initialize(re) } }.join
     end
 
+    assert_equal(Encoding::ASCII_8BIT, Regexp.new("b..", nil, "n").encoding)
     assert_equal("bar", "foobarbaz"[Regexp.new("b..", nil, "n")])
-    assert_equal(//n, Regexp.new("", nil, "n"))
 
     assert_raise(RegexpError) { Regexp.new(")(") }
   end
 
   def test_unescape
-    assert_raise(RegexpError) { s = '\\'; /#{ s }/ }
-    assert_equal(/\xFF/n, /#{ s="\\xFF" }/n)
+    assert_raise(ArgumentError) { s = '\\'; /#{ s }/ }
     assert_equal(/\177/, (s = '\177'; /#{ s }/))
+    assert_raise(ArgumentError) { s = '\u'; /#{ s }/ }
+    assert_raise(ArgumentError) { s = '\u{ ffffffff }'; /#{ s }/ }
+    assert_raise(ArgumentError) { s = '\u{ ffffff }'; /#{ s }/ }
+    assert_raise(ArgumentError) { s = '\u{ ffff X }'; /#{ s }/ }
+    assert_raise(ArgumentError) { s = '\u{ }'; /#{ s }/ }
+    assert_equal("b", "abc"[(s = '\u{0062}'; /#{ s }/)])
+    assert_equal("b", "abc"[(s = '\u0062'; /#{ s }/)])
+    assert_raise(ArgumentError) { s = '\u0'; /#{ s }/ }
+    assert_raise(ArgumentError) { s = '\u000X'; /#{ s }/ }
+    assert_raise(ArgumentError) { s = "\xff" + '\u3042'; /#{ s }/ }
+    assert_raise(ArgumentError) { s = '\u3042' + [0xff].pack("C"); /#{ s }/ }
+    assert_raise(SyntaxError) { s = ''; eval(%q(/\u#{ s }/)) }
 
-    assert_raise(RegexpError) { s = '\x'; /#{ s }/ }
+    assert_equal(/a/, eval(%q(s="\u0061";/#{s}/n)))
+    assert_raise(RegexpError) { s = "\u3042"; eval(%q(/#{s}/n)) }
+    assert_raise(RegexpError) { s = "\u0061"; eval(%q(/\u3042#{s}/n)) }
+    assert_raise(ArgumentError) { s1=[0xff].pack("C"); s2="\u3042"; eval(%q(/#{s1}#{s2}/)) }
+
+    assert_raise(ArgumentError) { s = '\x'; /#{ s }/ }
 
     assert_equal("\xe1", [0x00, 0xe1, 0xff].pack("C*")[/\M-a/])
     assert_equal("\xdc", [0x00, 0xdc, 0xff].pack("C*")[/\M-\\/])
@@ -175,16 +315,23 @@ class TestRegexp < Test::Unit::TestCase
     assert_equal("\x9b", [0x00, 0x9b, 0xff].pack("C*")[/\M-\e/])
     assert_equal("\x01", [0x00, 0x01, 0xff].pack("C*")[/\C-a/])
 
-    assert_raise(RegexpError) { s = '\M'; /#{ s }/ }
-    #assert_raise(RegexpError) { s = '\M-\M-a'; /#{ s }/ }
-    assert_raise(RegexpError) { s = '\M-\\'; /#{ s }/ }
+    assert_raise(ArgumentError) { s = '\M'; /#{ s }/ }
+    assert_raise(ArgumentError) { s = '\M-\M-a'; /#{ s }/ }
+    assert_raise(ArgumentError) { s = '\M-\\'; /#{ s }/ }
 
-    assert_raise(RegexpError) { s = '\C'; /#{ s }/ }
-    assert_raise(RegexpError) { s = '\c'; /#{ s }/ }
-    #assert_raise(RegexpError) { s = '\C-\C-a'; /#{ s }/ }
+    assert_raise(ArgumentError) { s = '\C'; /#{ s }/ }
+    assert_raise(ArgumentError) { s = '\c'; /#{ s }/ }
+    assert_raise(ArgumentError) { s = '\C-\C-a'; /#{ s }/ }
 
-    #assert_raise(RegexpError) { s = '\M-\z'; /#{ s }/ }
-    #assert_raise(RegexpError) { s = '\M-\777'; /#{ s }/ }
+    assert_raise(ArgumentError) { s = '\M-\z'; /#{ s }/ }
+    assert_raise(ArgumentError) { s = '\M-\777'; /#{ s }/ }
+
+    assert_equal("\u3042\u3042", "\u3042\u3042"[(s = "\u3042" + %q(\xe3\x81\x82); /#{s}/)])
+    assert_raise(ArgumentError) { s = "\u3042" + %q(\xe3); /#{s}/ }
+    assert_raise(ArgumentError) { s = "\u3042" + %q(\xe3\xe3); /#{s}/ }
+    assert_raise(ArgumentError) { s = '\u3042' + [0xff].pack("C"); /#{s}/ }
+
+    assert_raise(SyntaxError) { eval("/\u3042/n") }
 
     s = ".........."
     5.times { s.sub!(".", "") }
@@ -199,7 +346,17 @@ class TestRegexp < Test::Unit::TestCase
 
   def test_match
     assert_nil(//.match(nil))
+    assert_equal("abc", /.../.match(:abc)[0])
     assert_raise(TypeError) { /.../.match(Object.new)[0] }
+    assert_equal("bc", /../.match('abc', 1)[0])
+    assert_equal("bc", /../.match('abc', -2)[0])
+    assert_nil(/../.match("abc", -4))
+    assert_nil(/../.match("abc", 4))
+    assert_equal('\x', /../n.match("\u3042" + '\x', 1)[0])
+
+    r = nil
+    /.../.match("abc") {|m| r = m[0] }
+    assert_equal("abc", r)
 
     $_ = "abc"; assert_equal(1, ~/bc/)
     $_ = "abc"; assert_nil(~/d/)
@@ -217,14 +374,28 @@ class TestRegexp < Test::Unit::TestCase
     assert_equal("\\n", Regexp.quote("\n"))
     assert_equal("\\r", Regexp.quote("\r"))
     assert_equal("\\f", Regexp.quote("\f"))
+    assert_equal("\\v", Regexp.quote("\v"))
+    assert_equal("\u3042\\t", Regexp.quote("\u3042\t"))
     assert_equal("\\t\xff", Regexp.quote("\t" + [0xff].pack("C")))
   end
 
+  def test_try_convert
+    assert_equal(/re/, Regexp.try_convert(/re/))
+    assert_nil(Regexp.try_convert("re"))
+
+    o = Object.new
+    assert_nil(Regexp.try_convert(o))
+    def o.to_regexp() /foo/ end
+    assert_equal(/foo/, Regexp.try_convert(o))
+  end
+
   def test_union2
     assert_equal(/(?!)/, Regexp.union)
     assert_equal(/foo/, Regexp.union(/foo/))
     assert_equal(/foo/, Regexp.union([/foo/]))
     assert_equal(/\t/, Regexp.union("\t"))
+    assert_equal(/(?-mix:\u3042)|(?-mix:\u3042)/, Regexp.union(/\u3042/, /\u3042/))
+    assert_equal("\u3041", "\u3041"[Regexp.union(/\u3042/, "\u3041")])
   end
 
   def test_dup
@@ -238,6 +409,7 @@ class TestRegexp < Test::Unit::TestCase
     assert_equal(s, "X".sub!(/./, s))
     assert_equal('\\' + s, "X".sub!(/./, '\\' + s))
     assert_equal('\k', "foo".sub!(/.../, '\k'))
+    assert_raise(RuntimeError) { "foo".sub!(/(?<x>o)/, '\k<x') }
     assert_equal('foo[bar]baz', "foobarbaz".sub!(/(b..)/, '[\0]'))
     assert_equal('foo[foo]baz', "foobarbaz".sub!(/(b..)/, '[\`]'))
     assert_equal('foo[baz]baz', "foobarbaz".sub!(/(b..)/, '[\\\']'))
@@ -247,8 +419,8 @@ class TestRegexp < Test::Unit::TestCase
   end
 
   def test_KCODE
-    assert_nothing_raised { $KCODE = 'n' }
-    assert_equal('NONE', $KCODE)
+    assert_nil($KCODE)
+    assert_nothing_raised { $KCODE = nil }
     assert_equal(false, $=)
     assert_nothing_raised { $= = nil }
   end
@@ -284,6 +456,10 @@ class TestRegexp < Test::Unit::TestCase
     assert_equal("baz", $+)
   end
 
+  def test_rindex_regexp
+    assert_equal(3, "foobarbaz\u3042".rindex(/b../n, 5))
+  end
+
   def test_taint
     m = Thread.new do
       "foo"[/foo/]
@@ -291,10 +467,6 @@ class TestRegexp < Test::Unit::TestCase
       /foo/.match("foo")
     end.value
     assert(m.tainted?)
-    assert_nothing_raised('[ruby-core:26137]') {
-      m = proc {$SAFE = 4; %r"#{ }"o}.call
-    }
-    assert(m.tainted?)
   end
 
   def check(re, ss, fs = [])
@@ -302,16 +474,16 @@ class TestRegexp < Test::Unit::TestCase
     ss = [ss] unless ss.is_a?(Array)
     ss.each do |e, s|
       s ||= e
-      assert_match(re, s)
       m = re.match(s)
+      assert_kind_of(MatchData, m)
       assert_equal(e, m[0])
     end
     fs = [fs] unless fs.is_a?(Array)
-    fs.each {|s| assert_no_match(re, s) }
+    fs.each {|s| assert_nil(re.match(s)) }
   end
 
   def failcheck(re)
-    assert_raise(RegexpError) { %r"#{ re }" }
+    assert_raise(RegexpError) { /#{ re }/ }
   end
 
   def test_parse
@@ -320,16 +492,22 @@ class TestRegexp < Test::Unit::TestCase
     check(/\A.\b.\b.\B.\B.\z/, %w(a.aaa .a...), %w(aaaaa .....))
     check(/\A\s\S\z/, [' a', "\n."], ['  ', "\n\n", 'a '])
     check(/\A\d\D\z/, '0a', %w(00 aa))
+    check(/\A\h\H\z/, %w(0g ag BH), %w(a0 af GG))
     check(/\Afoo\Z\s\z/, "foo\n", ["foo", "foo\nbar"])
     assert_equal(%w(a b c), "abc def".scan(/\G\w/))
+    check(/\A\u3042\z/, "\u3042", ["", "\u3043", "a"])
     check(/\A(..)\1\z/, %w(abab ....), %w(abba aba))
+    failcheck('\1')
+    check(/\A\80\z/, "80", ["\100", ""])
     check(/\A\77\z/, "?")
     check(/\A\78\z/, "\7" + '8', ["\100", ""])
     check(/\A\Qfoo\E\z/, "QfooE")
-    failcheck('\Aa++\z')
+    check(/\Aa++\z/, "aaa")
     check('\Ax]\z', "x]")
     check(/x#foo/x, "x", "#foo")
     check(/\Ax#foo#{ "\n" }x\z/x, "xx", ["x", "x#foo\nx"])
+    check(/\A\p{Alpha}\z/, ["a", "z"], [".", "", ".."])
+    check(/\A\p{^Alpha}\z/, [".", "!"], ["!a", ""])
     check(/\A\n\z/, "\n")
     check(/\A\t\z/, "\t")
     check(/\A\r\z/, "\r")
@@ -337,39 +515,55 @@ class TestRegexp < Test::Unit::TestCase
     check(/\A\a\z/, "\007")
     check(/\A\e\z/, "\033")
     check(/\A\v\z/, "\v")
-    failcheck('(')
-    failcheck('(?foo)')
-    failcheck('/[1-\w]/')
   end
 
-  def test_exec
-    check(/A*B/, %w(B AB AAB AAAB), %w(A))
-    check(/\w*!/, %w(! a! ab! abc!), %w(abc))
-    check(/\w*\W/, %w(! a" ab# abc$), %w(abc))
-    check(/\w*\w/, %w(z az abz abcz), %w(!))
-    check(/[a-z]*\w/, %w(z az abz abcz), %w(!))
-    check(/[a-z]*\W/, %w(! a" ab# abc$), %w(A))
-    check(/((a|bb|ccc|dddd)(1|22|333|4444))/i, %w(a1 bb1 a22), %w(a2 b1))
-    check(/abc\B.\Bxyz/, %w(abcXxyz abc0xyz), %w(abc|xyz abc-xyz))
-    check(/\Bxyz/, [%w(xyz abcXxyz), %w(xyz abc0xyz)], %w(abc xyz abc-xyz))
-    check(/abc\B/, [%w(abc abcXxyz), %w(abc abc0xyz)], %w(abc xyz abc-xyz))
-    failcheck('(?<foo>abc)\1')
-    check(eval('/^(?:a?)?$/'), ["", "a"], ["aa"])
-    check(eval('/^(?:a+)?$/'), ["", "a", "aa"], ["ab"])
-    check(/^a??[ab]/, [["a", "a"], ["a", "aa"], ["b", "b"], ["a", "ab"]], ["c"])
-    check(/^(?:a*){3,5}$/, ["", "a", "aa", "aaa", "aaaa", "aaaaa", "aaaaaa"], ["b"])
-    check(/^(?:a+){3,5}$/, ["aaa", "aaaa", "aaaaa", "aaaaaa"], ["", "a", "aa", "b"])
+  def test_parse_kg
+    check(/\A(.)(.)\k<1>(.)\z/, %w(abac abab ....), %w(abcd aaba xxx))
+    check(/\A(.)(.)\k<-1>(.)\z/, %w(abbc abba ....), %w(abcd aaba xxx))
+    check(/\A(?<n>.)(?<x>\g<n>){0}(?<y>\k<n+0>){0}\g<x>\g<y>\z/, "aba", "abb")
+    check(/\A(?<n>.)(?<x>\g<n>){0}(?<y>\k<n+1>){0}\g<x>\g<y>\z/, "abb", "aba")
+    check(/\A(?<x>..)\k<x>\z/, %w(abab ....), %w(abac abba xxx))
+    check(/\A(.)(..)\g<-1>\z/, "abcde", %w(.... ......))
+    failcheck('\k<x>')
+    failcheck('\k<')
+    failcheck('\k<>')
+    failcheck('\k<.>')
+    failcheck('\k<x.>')
+    failcheck('\k<1.>')
+    failcheck('\k<x')
+    failcheck('\k<x+')
+    failcheck('()\k<-2>')
+    failcheck('()\g<-2>')
+    check(/\A(?<x>.)(?<x>.)\k<x>\z/, %w(aba abb), %w(abc .. ....))
+    check(/\k\g/, "kg")
   end
 
   def test_parse_curly_brace
+    check(/\A{/, ["{", ["{", "{x"]])
+    check(/\A{ /, ["{ ", ["{ ", "{ x"]])
+    check(/\A{,}\z/, "{,}")
+    check(/\A{}\z/, "{}")
     check(/\Aa{0}+\z/, "", %w(a aa aab))
     check(/\Aa{1}+\z/, %w(a aa), ["", "aab"])
     check(/\Aa{1,2}b{1,2}\z/, %w(ab aab abb aabb), ["", "aaabb", "abbb"])
-    check(/(?!x){0,1}/, [ ['', 'ab'], ['', ''] ])
     failcheck('.{100001}')
     failcheck('.{0,100001}')
     failcheck('.{1,0}')
     failcheck('{0}')
+    failcheck('(?!x){0,1}')
+  end
+
+  def test_parse_comment
+    check(/\A(?#foo\)bar)\z/, "", "a")
+    failcheck('(?#')
+  end
+
+  def test_char_type
+    check(/\u3042\d/, ["\u30421", "\u30422"])
+
+    # CClassTable cache test
+    assert(/\u3042\d/.match("\u30421"))
+    assert(/\u3042\d/.match("\u30422"))
   end
 
   def test_char_class
@@ -377,89 +571,44 @@ class TestRegexp < Test::Unit::TestCase
     failcheck('[x')
     check('\A[]]\z', "]", "")
     check('\A[]\.]+\z', %w(] . ]..]), ["", "["])
+    check(/\A[\u3042]\z/, "\u3042", "\u3042aa")
+    check(/\A[\u3042\x61]+\z/, ["aa\u3042aa", "\u3042\u3042", "a"], ["", "b"])
+    check(/\A[\u3042\x61\x62]+\z/, "abab\u3042abab\u3042")
     check(/\A[abc]+\z/, "abcba", ["", "ada"])
     check(/\A[\w][\W]\z/, %w(a. b!), %w(.. ab))
     check(/\A[\s][\S]\z/, [' a', "\n."], ['  ', "\n\n", 'a '])
     check(/\A[\d][\D]\z/, '0a', %w(00 aa))
+    check(/\A[\h][\H]\z/, %w(0g ag BH), %w(a0 af GG))
+    check(/\A[\p{Alpha}]\z/, ["a", "z"], [".", "", ".."])
+    check(/\A[\p{^Alpha}]\z/, [".", "!"], ["!a", ""])
     check(/\A[\xff]\z/, "\xff", ["", "\xfe"])
     check(/\A[\80]+\z/, "8008", ["\\80", "\100", "\1000"])
     check(/\A[\77]+\z/, "???")
     check(/\A[\78]+\z/, "\788\7")
     check(/\A[\0]\z/, "\0")
+    check(/\A[[:0]]\z/, [":", "0"], ["", ":0"])
     check(/\A[0-]\z/, ["0", "-"], "0-")
+    check('\A[a-&&\w]\z', "a", "-")
     check('\A[--0]\z', ["-", "/", "0"], ["", "1"])
     check('\A[\'--0]\z', %w(* + \( \) 0 ,), ["", ".", "1"])
     check(/\A[a-b-]\z/, %w(a b -), ["", "c"])
+    check('\A[a-b-&&\w]\z', %w(a b), ["", "-"])
+    check('\A[a-b-&&\W]\z', "-", ["", "a", "b"])
+    check('\A[a-c-e]\z', %w(a b c e), %w(- d)) # is it OK?
+    check(/\A[a-f&&[^b-c]&&[^e]]\z/, %w(a d f), %w(b c e g 0))
+    check(/\A[[^b-c]&&[^e]&&a-f]\z/, %w(a d f), %w(b c e g 0))
     check(/\A[\n\r\t]\z/, ["\n", "\r", "\t"])
-    failcheck('[9-1]')
-
-    assert_match(/\A\d+\z/, "0123456789")
-    assert_match(/\A\w+\z/, "09azAZ_")
-    #assert_match(/\A\s+\z/, "\r\n\v\f\r\s") # pending for 187, [Bug #1196]
   end
 
   def test_posix_bracket
     check(/\A[[:alpha:]0]\z/, %w(0 a), %w(1 .))
+    check(/\A[[:^alpha:]0]\z/, %w(0 1 .), "a")
+    check(/\A[[:alpha\:]]\z/, %w(a l p h a :), %w(b 0 1 .))
+    check(/\A[[:alpha:foo]0]\z/, %w(0 a), %w(1 .))
+    check(/\A[[:xdigit:]&&[:alpha:]]\z/, "a", %w(g 0))
     check('\A[[:abcdefghijklmnopqrstu:]]+\z', "[]")
     failcheck('[[:alpha')
     failcheck('[[:alpha:')
     failcheck('[[:alp:]]')
   end
-
-  def test_backward
-    assert_equal(3, "foobar".rindex(/b.r/i))
-    assert_equal(nil, "foovar".rindex(/b.r/i))
-    assert_equal(3, ("foo" + "bar" * 1000).rindex(/#{"bar"*1000}/))
-    assert_equal(4, ("foo\nbar\nbaz\n").rindex(/bar/i))
-  end
-
-  def test_uninitialized
-    assert_raise(TypeError) { Regexp.allocate.hash }
-    assert_raise(TypeError) { Regexp.allocate.eql? Regexp.allocate }
-    assert_raise(TypeError) { Regexp.allocate == Regexp.allocate }
-    assert_raise(TypeError) { Regexp.allocate =~ "" }
-    assert_equal(false, Regexp.allocate === Regexp.allocate)
-    assert_nil(~Regexp.allocate)
-    assert_raise(TypeError) { Regexp.allocate.match("") }
-    assert_raise(TypeError) { Regexp.allocate.to_s }
-    assert_raise(TypeError) { Regexp.allocate.inspect }
-    assert_raise(TypeError) { Regexp.allocate.source }
-    assert_raise(TypeError) { Regexp.allocate.casefold? }
-    assert_raise(TypeError) { Regexp.allocate.options }
-
-    assert_raise(TypeError) { MatchData.allocate.size }
-    assert_raise(TypeError) { MatchData.allocate.length }
-    assert_raise(TypeError) { MatchData.allocate.offset(0) }
-    assert_raise(TypeError) { MatchData.allocate.begin(0) }
-    assert_raise(TypeError) { MatchData.allocate.end(0) }
-    assert_raise(TypeError) { MatchData.allocate.to_a }
-    assert_raise(TypeError) { MatchData.allocate[:foo] }
-    assert_raise(TypeError) { MatchData.allocate.values_at }
-    assert_raise(TypeError) { MatchData.allocate.pre_match }
-    assert_raise(TypeError) { MatchData.allocate.post_match }
-    assert_raise(TypeError) { MatchData.allocate.to_s }
-    assert_raise(TypeError) { MatchData.allocate.string }
-    $~ = MatchData.allocate
-    assert_raise(TypeError) { $& }
-    assert_raise(TypeError) { $` }
-    assert_raise(TypeError) { $' }
-    assert_raise(TypeError) { $+ }
-  end
-
-  def test_regexp_poped
-    assert_nothing_raised { eval("a = 1; /\#{ a }/; a") }
-    assert_nothing_raised { eval("a = 1; /\#{ a }/o; a") }
-  end
-
-  def test_optimize_last_anycharstar
-    s = "1" + " " * 5000000
-    assert_nothing_raised { s.match(/(\d) (.*)/) }
-    assert_equal("1", $1)
-    assert_equal(" " * 4999999, $2)
-  end
-
-  def test_range_greedy
-    /wo{0,3}?/ =~ "woo"
-    assert_equal("w", $&)
-  end
 end
